@@ -27,17 +27,14 @@ attack objective.
 from __future__ import absolute_import, division, print_function, unicode_literals
 
 import logging
-from typing import Optional, Tuple
+from typing import Optional, Tuple, TYPE_CHECKING
 
 import numpy as np
 from tqdm import trange
 
 from art.config import ART_NUMPY_DTYPE
 from art.estimators.estimator import BaseEstimator
-from art.estimators.classification.classifier import (
-    ClassGradientsMixin,
-    ClassifierGradients,
-)
+from art.estimators.classification.classifier import ClassGradientsMixin
 from art.attacks.attack import EvasionAttack
 from art.utils import (
     compute_success,
@@ -47,6 +44,9 @@ from art.utils import (
 )
 from art.utils import check_and_transform_label_format
 
+if TYPE_CHECKING:
+    from art.utils import CLASSIFIER_CLASS_LOSS_GRADIENTS_TYPE
+
 logger = logging.getLogger(__name__)
 
 
@@ -76,7 +76,7 @@ class CarliniL2Method(EvasionAttack):
 
     def __init__(
         self,
-        classifier: ClassifierGradients,
+        classifier: "CLASSIFIER_CLASS_LOSS_GRADIENTS_TYPE",
         confidence: float = 0.0,
         targeted: bool = False,
         learning_rate: float = 0.01,
@@ -108,12 +108,12 @@ class CarliniL2Method(EvasionAttack):
         :param max_halving: Maximum number of halving steps in the line search optimization.
         :param max_doubling: Maximum number of doubling steps in the line search optimization.
         :param batch_size: Size of the batch on which adversarial samples are generated.
-        :param verbose: Indicates whether to print verbose messages.
+        :param verbose: Show progress bars.
         """
-        super(CarliniL2Method, self).__init__(estimator=classifier)
+        super().__init__(estimator=classifier)
 
         self.confidence = confidence
-        self.targeted = targeted
+        self._targeted = targeted
         self.learning_rate = learning_rate
         self.binary_search_steps = binary_search_steps
         self.max_iter = max_iter
@@ -498,7 +498,7 @@ class CarliniLInfMethod(EvasionAttack):
 
     def __init__(
         self,
-        classifier: ClassifierGradients,
+        classifier: "CLASSIFIER_CLASS_LOSS_GRADIENTS_TYPE",
         confidence: float = 0.0,
         targeted: bool = False,
         learning_rate: float = 0.01,
@@ -523,12 +523,12 @@ class CarliniLInfMethod(EvasionAttack):
         :param max_doubling: Maximum number of doubling steps in the line search optimization.
         :param eps: An upper bound for the L_0 norm of the adversarial perturbation.
         :param batch_size: Size of the batch on which adversarial samples are generated.
-        :param verbose: Indicates whether to print verbose messages.
+        :param verbose: Show progress bars.
         """
-        super(CarliniLInfMethod, self).__init__(estimator=classifier)
+        super().__init__(estimator=classifier)
 
         self.confidence = confidence
-        self.targeted = targeted
+        self._targeted = targeted
         self.learning_rate = learning_rate
         self.max_iter = max_iter
         self.max_halving = max_halving
