@@ -23,7 +23,7 @@ This module implements the elastic net attack `ElasticNet`. This is a white-box
 from __future__ import absolute_import, division, print_function, unicode_literals
 
 import logging
-from typing import Optional, Tuple
+from typing import Optional, Tuple, TYPE_CHECKING
 
 import numpy as np
 import six
@@ -31,16 +31,17 @@ from tqdm import trange
 
 from art.config import ART_NUMPY_DTYPE
 from art.attacks.attack import EvasionAttack
-from art.estimators.classification.classifier import (
-    ClassGradientsMixin,
-    ClassifierGradients,
-)
+from art.estimators.estimator import BaseEstimator
+from art.estimators.classification.classifier import ClassGradientsMixin
 from art.utils import (
     compute_success,
     get_labels_np_array,
     check_and_transform_label_format,
 )
 
+if TYPE_CHECKING:
+    from art.utils import CLASSIFIER_CLASS_LOSS_GRADIENTS_TYPE
+
 logger = logging.getLogger(__name__)
 
 
@@ -61,13 +62,14 @@ class ElasticNet(EvasionAttack):
         "initial_const",
         "batch_size",
         "decision_rule",
+        "verbose",
     ]
 
-    _estimator_requirements = (ClassGradientsMixin,)
+    _estimator_requirements = (BaseEstimator, ClassGradientsMixin)
 
     def __init__(
         self,
-        classifier: ClassifierGradients,
+        classifier: "CLASSIFIER_CLASS_LOSS_GRADIENTS_TYPE",
         confidence: float = 0.0,
         targeted: bool = False,
         learning_rate: float = 1e-2,
@@ -77,6 +79,7 @@ class ElasticNet(EvasionAttack):
         initial_const: float = 1e-3,
         batch_size: int = 1,
         decision_rule: str = "EN",
+        verbose: bool = True,
     ) -> None:
         """
         Create an ElasticNet attack instance.
@@ -95,10 +98,11 @@ class ElasticNet(EvasionAttack):
                Carlini and Wagner (2016).
         :param batch_size: Internal size of batches on which adversarial samples are generated.
         :param decision_rule: Decision rule. 'EN' means Elastic Net rule, 'L1' means L1 rule, 'L2' means L2 rule.
+        :param verbose: Show progress bars.
         """
-        super(ElasticNet, self).__init__(estimator=classifier)
+        super().__init__(estimator=classifier)
         self.confidence = confidence
-        self.targeted = targeted
+        self._targeted = targeted
         self.learning_rate = learning_rate
         self.binary_search_steps = binary_search_steps
         self.max_iter = max_iter
@@ -106,6 +110,7 @@ class ElasticNet(EvasionAttack):
         self.initial_const = initial_const
         self.batch_size = batch_size
         self.decision_rule = decision_rule
+        self.verbose = verbose
         self._check_params()
 
     def _loss(self, x: np.ndarray, x_adv: np.ndarray) -> tuple:
@@ -200,7 +205,7 @@ class ElasticNet(EvasionAttack):
 
         # Compute adversarial examples with implicit batching
         nb_batches = int(np.ceil(x_adv.shape[0] / float(self.batch_size)))
-        for batch_id in trange(nb_batches, desc="EAD"):
+        for batch_id in trange(nb_batches, desc="EAD", disable=not self.verbose):
             batch_index_1, batch_index_2 = batch_id * self.batch_size, (batch_id + 1) * self.batch_size
             x_batch = x_adv[batch_index_1:batch_index_2]
             y_batch = y[batch_index_1:batch_index_2]
@@ -385,3 +390,6 @@ class ElasticNet(EvasionAttack):
 
         if not isinstance(self.decision_rule, six.string_types) or self.decision_rule not in ["EN", "L1", "L2"]:
             raise ValueError("The decision rule only supports `EN`, `L1`, `L2`.")
+
+        if not isinstance(self.verbose, bool):
+            raise ValueError("The argument `verbose` has to be of type bool.")
