@@ -24,17 +24,14 @@ Method attack and extends it to other norms, therefore it is called the Fast Gra
 from __future__ import absolute_import, division, print_function, unicode_literals
 
 import logging
-from typing import Optional
+from typing import Optional, Union, TYPE_CHECKING
 
 import numpy as np
 
 from art.config import ART_NUMPY_DTYPE
 from art.attacks.attack import EvasionAttack
 from art.estimators.estimator import BaseEstimator, LossGradientsMixin
-from art.estimators.classification.classifier import (
-    ClassifierGradients,
-    ClassifierMixin,
-)
+from art.estimators.classification.classifier import ClassifierMixin
 from art.utils import (
     compute_success,
     get_labels_np_array,
@@ -43,6 +40,9 @@ from art.utils import (
     check_and_transform_label_format,
 )
 
+if TYPE_CHECKING:
+    from art.utils import CLASSIFIER_LOSS_GRADIENTS_TYPE
+
 logger = logging.getLogger(__name__)
 
 
@@ -68,10 +68,10 @@ class FastGradientMethod(EvasionAttack):
 
     def __init__(
         self,
-        estimator: ClassifierGradients,
-        norm: int = np.inf,
-        eps: float = 0.3,
-        eps_step: float = 0.1,
+        estimator: "CLASSIFIER_LOSS_GRADIENTS_TYPE",
+        norm: Union[int, float, str] = np.inf,
+        eps: Union[int, float, np.ndarray] = 0.3,
+        eps_step: Union[int, float, np.ndarray] = 0.1,
         targeted: bool = False,
         num_random_init: int = 0,
         batch_size: int = 32,
@@ -81,7 +81,7 @@ class FastGradientMethod(EvasionAttack):
         Create a :class:`.FastGradientMethod` instance.
 
         :param estimator: A trained classifier.
-        :param norm: The norm of the adversarial perturbation. Possible values: np.inf, 1 or 2.
+        :param norm: The norm of the adversarial perturbation. Possible values: "inf", np.inf, 1 or 2.
         :param eps: Attack step size (input variation).
         :param eps_step: Step size of input variation for minimal perturbation computation.
         :param targeted: Indicates whether the attack is targeted (True) or untargeted (False)
@@ -91,17 +91,28 @@ class FastGradientMethod(EvasionAttack):
         :param minimal: Indicates if computing the minimal perturbation (True). If True, also define `eps_step` for
                         the step size and eps for the maximum perturbation.
         """
-        super(FastGradientMethod, self).__init__(estimator=estimator)
+        super().__init__(estimator=estimator)
         self.norm = norm
         self.eps = eps
         self.eps_step = eps_step
-        self.targeted = targeted
+        self._targeted = targeted
         self.num_random_init = num_random_init
         self.batch_size = batch_size
         self.minimal = minimal
         self._project = True
         FastGradientMethod._check_params(self)
 
+    def _check_compatibility_input_and_eps(self, x: np.ndarray):
+        """
+        Check the compatibility of the input with `eps` and `eps_step` which are of the same shape.
+
+        :param x: An array with the original inputs.
+        """
+        if isinstance(self.eps, np.ndarray):
+            # Ensure the eps array is broadcastable
+            if self.eps.ndim > x.ndim:
+                raise ValueError("The `eps` shape must be broadcastable to input shape.")
+
     def _minimal_perturbation(self, x: np.ndarray, y: np.ndarray, mask: np.ndarray) -> np.ndarray:
         """
         Iteratively compute the minimal perturbation necessary to make the class prediction change. Stop when the
@@ -134,20 +145,47 @@ class FastGradientMethod(EvasionAttack):
 
             # Get current predictions
             active_indices = np.arange(len(batch))
-            current_eps = self.eps_step
-            while active_indices.size > 0 and current_eps <= self.eps:
+
+            if isinstance(self.eps, np.ndarray):
+                if len(self.eps.shape) == len(x.shape) and self.eps.shape[0] == x.shape[0]:
+                    current_eps = self.eps_step[batch_index_1:batch_index_2]
+                    partial_stop_condition = (current_eps <= self.eps[batch_index_1:batch_index_2]).all()
+
+                else:
+                    current_eps = self.eps_step
+                    partial_stop_condition = (current_eps <= self.eps).all()
+
+            else:
+                current_eps = self.eps_step
+                partial_stop_condition = current_eps <= self.eps
+
+            while active_indices.size > 0 and partial_stop_condition:
                 # Adversarial crafting
                 current_x = self._apply_perturbation(x[batch_index_1:batch_index_2], perturbation, current_eps)
+
                 # Update
                 batch[active_indices] = current_x[active_indices]
                 adv_preds = self.estimator.predict(batch)
+
                 # If targeted active check to see whether we have hit the target, otherwise head to anything but
                 if self.targeted:
                     active_indices = np.where(np.argmax(batch_labels, axis=1) != np.argmax(adv_preds, axis=1))[0]
                 else:
                     active_indices = np.where(np.argmax(batch_labels, axis=1) == np.argmax(adv_preds, axis=1))[0]
 
-                current_eps += self.eps_step
+                # Update current eps and check the stop condition
+                if isinstance(self.eps, np.ndarray):
+                    if len(self.eps.shape) == len(x.shape) and self.eps.shape[0] == x.shape[0]:
+                        current_eps = current_eps + self.eps_step[batch_index_1:batch_index_2]
+                        partial_stop_condition = (current_eps <= self.eps[batch_index_1:batch_index_2]).all()
+
+                    else:
+                        current_eps = current_eps + self.eps_step
+                        partial_stop_condition = (current_eps <= self.eps).all()
+
+                else:
+                    current_eps = current_eps + self.eps_step
+                    partial_stop_condition = current_eps <= self.eps
 
             adv_x[batch_index_1:batch_index_2] = batch
 
@@ -161,12 +199,17 @@ class FastGradientMethod(EvasionAttack):
                   (nb_samples,). Only provide this parameter if you'd like to use true labels when crafting adversarial
                   samples. Otherwise, model predictions are used as labels to avoid the "label leaking" effect
                   (explained in this paper: https://arxiv.org/abs/1611.01236). Default is `None`.
-        :param mask: An array with a mask to be applied to the adversarial perturbations. Shape needs to be
-                     broadcastable to the shape of x. Any features for which the mask is zero will not be adversarially
-                     perturbed.
+        :param mask: An array with a mask broadcastable to input `x` defining where to apply adversarial perturbations.
+                     Shape needs to be broadcastable to the shape of x and can also be of the same shape as `x`. Any
+                     features for which the mask is zero will not be adversarially perturbed.
         :type mask: `np.ndarray`
         :return: An array holding the adversarial examples.
         """
+        mask = self._get_mask(x, **kwargs)
+
+        # Ensure eps is broadcastable
+        self._check_compatibility_input_and_eps(x=x)
+
         if isinstance(self.estimator, ClassifierMixin):
             y = check_and_transform_label_format(y, self.estimator.nb_classes)
 
@@ -177,17 +220,9 @@ class FastGradientMethod(EvasionAttack):
 
                 # Use model predictions as correct outputs
                 logger.info("Using model predictions as correct labels for FGM.")
-                y = get_labels_np_array(
-                    self.estimator.predict(x, batch_size=self.batch_size)  # type: ignore
-                )
+                y = get_labels_np_array(self.estimator.predict(x, batch_size=self.batch_size))  # type: ignore
             y = y / np.sum(y, axis=1, keepdims=True)
 
-            mask = kwargs.get("mask")
-            if mask is not None:
-                # ensure the mask is broadcastable:
-                if len(mask.shape) > len(x.shape) or mask.shape != x.shape[-len(mask.shape) :]:
-                    raise ValueError("mask shape must be broadcastable to input shape")
-
             # Return adversarial examples computed with minimal perturbation if option is active
             rate_best: Optional[float]
             if self.minimal:
@@ -232,9 +267,6 @@ class FastGradientMethod(EvasionAttack):
             if self.minimal:
                 raise ValueError("Minimal perturbation is only supported for classification.")
 
-            if kwargs.get("mask") is not None:
-                raise ValueError("Mask is only supported for classification.")
-
             if y is None:
                 # Throw error if attack is targeted, but no targets are provided
                 if self.targeted:
@@ -250,14 +282,40 @@ class FastGradientMethod(EvasionAttack):
 
     def _check_params(self) -> None:
         # Check if order of the norm is acceptable given current implementation
-        if self.norm not in [np.inf, int(1), int(2)]:
-            raise ValueError("Norm order must be either `np.inf`, 1, or 2.")
+        if self.norm not in [1, 2, np.inf, "inf"]:
+            raise ValueError('Norm order must be either 1, 2, `np.inf` or "inf".')
 
-        if self.eps <= 0:
-            raise ValueError("The perturbation size `eps` has to be positive.")
+        if (not (isinstance(self.eps, (int, float, np.ndarray)) and isinstance(self.eps_step, (int, float)))) and (
+            hasattr(self, "minimal")
+            and self.minimal
+            and not (isinstance(self.eps, np.ndarray) and isinstance(self.eps_step, np.ndarray))
+        ):
+            raise TypeError(
+                "The perturbation size `eps` and the perturbation step-size `eps_step` must have the same type."
+            )
 
-        if self.eps_step <= 0:
-            raise ValueError("The perturbation step-size `eps_step` has to be positive.")
+        if isinstance(self.eps, (int, float)):
+            if self.eps <= 0:
+                raise ValueError("The perturbation size `eps` has to be positive.")
+        else:
+            if (self.eps <= 0).any():
+                raise ValueError("The perturbation size `eps` has to be positive.")
+
+        if isinstance(self.eps_step, (int, float)):
+            if self.eps_step <= 0:
+                raise ValueError("The perturbation step-size `eps_step` has to be positive.")
+        else:
+            if (self.eps_step <= 0).any():
+                raise ValueError("The perturbation step-size `eps_step` has to be positive.")
+
+        if (
+            isinstance(self.eps, np.ndarray)
+            and isinstance(self.eps_step, np.ndarray)
+            and self.eps.shape != self.eps_step.shape
+        ):
+            raise ValueError(
+                "The perturbation size `eps` and the perturbation step-size `eps_step` must have the same shape."
+            )
 
         if not isinstance(self.targeted, bool):
             raise ValueError("The flag `targeted` has to be of type bool.")
@@ -274,30 +332,51 @@ class FastGradientMethod(EvasionAttack):
         if not isinstance(self.minimal, bool):
             raise ValueError("The flag `minimal` has to be of type bool.")
 
-    def _compute_perturbation(self, batch: np.ndarray, batch_labels: np.ndarray, mask: np.ndarray) -> np.ndarray:
+    def _compute_perturbation(
+        self, batch: np.ndarray, batch_labels: np.ndarray, mask: Optional[np.ndarray]
+    ) -> np.ndarray:
         # Pick a small scalar to avoid division by 0
         tol = 10e-8
 
         # Get gradient wrt loss; invert it if attack is targeted
         grad = self.estimator.loss_gradient(batch, batch_labels) * (1 - 2 * int(self.targeted))
 
+        # Apply mask
+        if mask is not None:
+            grad = np.where(mask == 0.0, 0.0, grad)
+
         # Apply norm bound
-        if self.norm == np.inf:
-            grad = np.sign(grad)
-        elif self.norm == 1:
-            ind = tuple(range(1, len(batch.shape)))
-            grad = grad / (np.sum(np.abs(grad), axis=ind, keepdims=True) + tol)
-        elif self.norm == 2:
-            ind = tuple(range(1, len(batch.shape)))
-            grad = grad / (np.sqrt(np.sum(np.square(grad), axis=ind, keepdims=True)) + tol)
+        def _apply_norm(grad, object_type=False):
+            if self.norm in [np.inf, "inf"]:
+                grad = np.sign(grad)
+            elif self.norm == 1:
+                if not object_type:
+                    ind = tuple(range(1, len(batch.shape)))
+                else:
+                    ind = None
+                grad = grad / (np.sum(np.abs(grad), axis=ind, keepdims=True) + tol)
+            elif self.norm == 2:
+                if not object_type:
+                    ind = tuple(range(1, len(batch.shape)))
+                else:
+                    ind = None
+                grad = grad / (np.sqrt(np.sum(np.square(grad), axis=ind, keepdims=True)) + tol)
+            return grad
+
+        if batch.dtype == np.object:
+            for i_sample in range(batch.shape[0]):
+                grad[i_sample] = _apply_norm(grad[i_sample], object_type=True)
+                assert batch[i_sample].shape == grad[i_sample].shape
+        else:
+            grad = _apply_norm(grad)
+
         assert batch.shape == grad.shape
 
-        if mask is None:
-            return grad
-        else:
-            return grad * (mask.astype(ART_NUMPY_DTYPE))
+        return grad
 
-    def _apply_perturbation(self, batch: np.ndarray, perturbation: np.ndarray, eps_step: float) -> np.ndarray:
+    def _apply_perturbation(
+        self, batch: np.ndarray, perturbation: np.ndarray, eps_step: Union[int, float, np.ndarray]
+    ) -> np.ndarray:
         batch = batch + eps_step * perturbation
 
         if self.estimator.clip_values is not None:
@@ -311,15 +390,15 @@ class FastGradientMethod(EvasionAttack):
         x: np.ndarray,
         x_init: np.ndarray,
         y: np.ndarray,
-        mask: np.ndarray,
-        eps: float,
-        eps_step: float,
+        mask: Optional[np.ndarray],
+        eps: Union[int, float, np.ndarray],
+        eps_step: Union[int, float, np.ndarray],
         project: bool,
         random_init: bool,
     ) -> np.ndarray:
         if random_init:
             n = x.shape[0]
-            m = np.prod(x.shape[1:])
+            m = np.prod(x.shape[1:]).item()
             random_perturbation = random_sphere(n, m, eps, self.norm).reshape(x.shape).astype(ART_NUMPY_DTYPE)
             if mask is not None:
                 random_perturbation = random_perturbation * (mask.astype(ART_NUMPY_DTYPE))
@@ -329,11 +408,15 @@ class FastGradientMethod(EvasionAttack):
                 clip_min, clip_max = self.estimator.clip_values
                 x_adv = np.clip(x_adv, clip_min, clip_max)
         else:
-            x_adv = x.astype(ART_NUMPY_DTYPE)
+            if x.dtype == np.object:
+                x_adv = x.copy()
+            else:
+                x_adv = x.astype(ART_NUMPY_DTYPE)
 
-            # Compute perturbation with implicit batching
+        # Compute perturbation with implicit batching
         for batch_id in range(int(np.ceil(x.shape[0] / float(self.batch_size)))):
             batch_index_1, batch_index_2 = batch_id * self.batch_size, (batch_id + 1) * self.batch_size
+            batch_index_2 = min(batch_index_2, x.shape[0])
             batch = x_adv[batch_index_1:batch_index_2]
             batch_labels = y[batch_index_1:batch_index_2]
 
@@ -347,13 +430,72 @@ class FastGradientMethod(EvasionAttack):
             # Get perturbation
             perturbation = self._compute_perturbation(batch, batch_labels, mask_batch)
 
+            # Compute batch_eps and batch_eps_step
+            if isinstance(eps, np.ndarray):
+                if len(eps.shape) == len(x.shape) and eps.shape[0] == x.shape[0]:
+                    batch_eps = eps[batch_index_1:batch_index_2]
+                    batch_eps_step = eps_step[batch_index_1:batch_index_2]
+
+                else:
+                    batch_eps = eps
+                    batch_eps_step = eps_step
+
+            else:
+                batch_eps = eps
+                batch_eps_step = eps_step
+
             # Apply perturbation and clip
-            x_adv[batch_index_1:batch_index_2] = self._apply_perturbation(batch, perturbation, eps_step)
+            x_adv[batch_index_1:batch_index_2] = self._apply_perturbation(batch, perturbation, batch_eps_step)
 
             if project:
-                perturbation = projection(
-                    x_adv[batch_index_1:batch_index_2] - x_init[batch_index_1:batch_index_2], eps, self.norm
-                )
-                x_adv[batch_index_1:batch_index_2] = x_init[batch_index_1:batch_index_2] + perturbation
+                if x_adv.dtype == np.object:
+                    for i_sample in range(batch_index_1, batch_index_2):
+                        if isinstance(batch_eps, np.ndarray) and batch_eps.shape[0] == x_adv.shape[0]:
+                            perturbation = projection(
+                                x_adv[i_sample] - x_init[i_sample], batch_eps[i_sample], self.norm
+                            )
+
+                        else:
+                            perturbation = projection(x_adv[i_sample] - x_init[i_sample], batch_eps, self.norm)
+
+                        x_adv[i_sample] = x_init[i_sample] + perturbation
+
+                else:
+                    perturbation = projection(
+                        x_adv[batch_index_1:batch_index_2] - x_init[batch_index_1:batch_index_2], batch_eps, self.norm
+                    )
+                    x_adv[batch_index_1:batch_index_2] = x_init[batch_index_1:batch_index_2] + perturbation
 
         return x_adv
+
+    @staticmethod
+    def _get_mask(x: np.ndarray, **kwargs) -> np.ndarray:
+        """
+        Get the mask from the kwargs.
+
+        :param x: An array with the original inputs.
+        :param mask: An array with a mask to be applied to the adversarial perturbations. Shape needs to be
+                     broadcastable to the shape of x. Any features for which the mask is zero will not be adversarially
+                     perturbed.
+        :type mask: `np.ndarray`
+        :return: The mask.
+        """
+        mask = kwargs.get("mask")
+
+        if mask is not None:
+            if mask.ndim > x.ndim:
+                raise ValueError("Mask shape must be broadcastable to input shape.")
+
+            if not (np.issubdtype(mask.dtype, np.floating) or mask.dtype == np.bool):
+                raise ValueError(
+                    "The `mask` has to be either of type np.float32, np.float64 or np.bool. The provided"
+                    "`mask` is of type {}.".format(mask.dtype)
+                )
+
+            if np.issubdtype(mask.dtype, np.floating) and np.amin(mask) < 0.0:
+                raise ValueError(
+                    "The `mask` of type np.float32 or np.float64 requires all elements to be either zero"
+                    "or positive values."
+                )
+
+        return mask
