@@ -36,7 +36,7 @@ from art.estimators.classification import ClassifierMixin
 from art.utils import compute_success, to_categorical, check_and_transform_label_format
 
 if TYPE_CHECKING:
-    from art.estimators.classification.classifier import Classifier
+    from art.utils import CLASSIFIER_TYPE
 
 logger = logging.getLogger(__name__)
 
@@ -58,32 +58,35 @@ class HopSkipJump(EvasionAttack):
         "init_size",
         "curr_iter",
         "batch_size",
+        "verbose",
     ]
     _estimator_requirements = (BaseEstimator, ClassifierMixin)
 
     def __init__(
         self,
-        classifier: "Classifier",
+        classifier: "CLASSIFIER_TYPE",
         targeted: bool = False,
-        norm: int = 2,
+        norm: Union[int, float, str] = 2,
         max_iter: int = 50,
         max_eval: int = 10000,
         init_eval: int = 100,
         init_size: int = 100,
+        verbose: bool = True,
     ) -> None:
         """
         Create a HopSkipJump attack instance.
 
         :param classifier: A trained classifier.
         :param targeted: Should the attack target one specific class.
-        :param norm: Order of the norm. Possible values: np.inf or 2.
+        :param norm: Order of the norm. Possible values: "inf", np.inf or 2.
         :param max_iter: Maximum number of iterations.
         :param max_eval: Maximum number of evaluations for estimating gradient.
         :param init_eval: Initial number of evaluations for estimating gradient.
         :param init_size: Maximum number of trials for initial generation of adversarial examples.
+        :param verbose: Show progress bars.
         """
-        super(HopSkipJump, self).__init__(estimator=classifier)
-        self.targeted = targeted
+        super().__init__(estimator=classifier)
+        self._targeted = targeted
         self.norm = norm
         self.max_iter = max_iter
         self.max_eval = max_eval
@@ -91,6 +94,7 @@ class HopSkipJump(EvasionAttack):
         self.init_size = init_size
         self.curr_iter = 0
         self.batch_size = 1
+        self.verbose = verbose
         self._check_params()
         self.curr_iter = 0
 
@@ -107,12 +111,18 @@ class HopSkipJump(EvasionAttack):
         :param x: An array with the original inputs to be attacked.
         :param y: Target values (class labels) one-hot-encoded of shape `(nb_samples, nb_classes)` or indices of shape
                   (nb_samples,).
+        :param mask: An array with a mask broadcastable to input `x` defining where to apply adversarial perturbations.
+                     Shape needs to be broadcastable to the shape of x and can also be of the same shape as `x`. Any
+                     features for which the mask is zero will not be adversarially perturbed.
+        :type mask: `np.ndarray`
         :param x_adv_init: Initial array to act as initial adversarial examples. Same shape as `x`.
         :type x_adv_init: `np.ndarray`
         :param resume: Allow users to continue their previous attack.
         :type resume: `bool`
         :return: An array holding the adversarial examples.
         """
+        mask = kwargs.get("mask")
+
         y = check_and_transform_label_format(y, self.estimator.nb_classes)
 
         # Check whether users need a stateful attack
@@ -123,6 +133,15 @@ class HopSkipJump(EvasionAttack):
         else:
             start = 0
 
+        # Check the mask
+        if mask is not None:
+            if len(mask.shape) == len(x.shape):
+                mask = mask.astype(ART_NUMPY_DTYPE)
+            else:
+                mask = np.array([mask.astype(ART_NUMPY_DTYPE)] * x.shape[0])
+        else:
+            mask = np.array([None] * x.shape[0])
+
         # Get clip_min and clip_max from the classifier or infer them from data
         if self.estimator.clip_values is not None:
             clip_min, clip_max = self.estimator.clip_values
@@ -136,7 +155,14 @@ class HopSkipJump(EvasionAttack):
         x_adv_init = kwargs.get("x_adv_init")
 
         if x_adv_init is not None:
+            # Add mask param to the x_adv_init
+            for i in range(x.shape[0]):
+                if mask[i] is not None:
+                    x_adv_init[i] = x_adv_init[i] * mask[i] + x[i] * (1 - mask[i])
+
+            # Do prediction on the init
             init_preds = np.argmax(self.estimator.predict(x_adv_init, batch_size=self.batch_size), axis=1)
+
         else:
             init_preds = [None] * len(x)
             x_adv_init = [None] * len(x)
@@ -147,12 +173,14 @@ class HopSkipJump(EvasionAttack):
 
         # Some initial setups
         x_adv = x.astype(ART_NUMPY_DTYPE)
+
         if y is not None:
             y = np.argmax(y, axis=1)
 
         # Generate the adversarial samples
-        for ind, val in enumerate(tqdm(x_adv, desc="HopSkipJump")):
+        for ind, val in enumerate(tqdm(x_adv, desc="HopSkipJump", disable=not self.verbose)):
             self.curr_iter = start
+
             if self.targeted:
                 x_adv[ind] = self._perturb(
                     x=val,
@@ -160,9 +188,11 @@ class HopSkipJump(EvasionAttack):
                     y_p=preds[ind],
                     init_pred=init_preds[ind],
                     adv_init=x_adv_init[ind],
+                    mask=mask[ind],
                     clip_min=clip_min,
                     clip_max=clip_max,
                 )
+
             else:
                 x_adv[ind] = self._perturb(
                     x=val,
@@ -170,6 +200,7 @@ class HopSkipJump(EvasionAttack):
                     y_p=preds[ind],
                     init_pred=init_preds[ind],
                     adv_init=x_adv_init[ind],
+                    mask=mask[ind],
                     clip_min=clip_min,
                     clip_max=clip_max,
                 )
@@ -185,7 +216,15 @@ class HopSkipJump(EvasionAttack):
         return x_adv
 
     def _perturb(
-        self, x: np.ndarray, y: int, y_p: int, init_pred: int, adv_init: np.ndarray, clip_min: float, clip_max: float,
+        self,
+        x: np.ndarray,
+        y: int,
+        y_p: int,
+        init_pred: int,
+        adv_init: np.ndarray,
+        mask: Optional[np.ndarray],
+        clip_min: float,
+        clip_max: float,
     ) -> np.ndarray:
         """
         Internal attack function for one example.
@@ -195,24 +234,35 @@ class HopSkipJump(EvasionAttack):
         :param y_p: The predicted label of x.
         :param init_pred: The predicted label of the initial image.
         :param adv_init: Initial array to act as an initial adversarial example.
+        :param mask: An array with a mask to be applied to the adversarial perturbations. Shape needs to be
+                     broadcastable to the shape of x. Any features for which the mask is zero will not be adversarially
+                     perturbed.
         :param clip_min: Minimum value of an example.
         :param clip_max: Maximum value of an example.
         :return: An adversarial example.
         """
         # First, create an initial adversarial sample
-        initial_sample = self._init_sample(x, y, y_p, init_pred, adv_init, clip_min, clip_max)
+        initial_sample = self._init_sample(x, y, y_p, init_pred, adv_init, mask, clip_min, clip_max)
 
         # If an initial adversarial example is not found, then return the original image
         if initial_sample is None:
             return x
 
-        # If an initial adversarial example found, then go with hopskipjump attack
-        x_adv = self._attack(initial_sample[0], x, initial_sample[1], clip_min, clip_max)
+        # If an initial adversarial example found, then go with HopSkipJump attack
+        x_adv = self._attack(initial_sample[0], x, initial_sample[1], mask, clip_min, clip_max)
 
         return x_adv
 
     def _init_sample(
-        self, x: np.ndarray, y: int, y_p: int, init_pred: int, adv_init: np.ndarray, clip_min: float, clip_max: float,
+        self,
+        x: np.ndarray,
+        y: int,
+        y_p: int,
+        init_pred: int,
+        adv_init: np.ndarray,
+        mask: Optional[np.ndarray],
+        clip_min: float,
+        clip_max: float,
     ) -> Optional[Union[np.ndarray, Tuple[np.ndarray, int]]]:
         """
         Find initial adversarial example for the attack.
@@ -222,6 +272,9 @@ class HopSkipJump(EvasionAttack):
         :param y_p: The predicted label of x.
         :param init_pred: The predicted label of the initial image.
         :param adv_init: Initial array to act as an initial adversarial example.
+        :param mask: An array with a mask to be applied to the adversarial perturbations. Shape needs to be
+                     broadcastable to the shape of x. Any features for which the mask is zero will not be adversarially
+                     perturbed.
         :param clip_min: Minimum value of an example.
         :param clip_max: Maximum value of an example.
         :return: An adversarial example.
@@ -241,6 +294,10 @@ class HopSkipJump(EvasionAttack):
             # Attack unsatisfied yet and the initial image unsatisfied
             for _ in range(self.init_size):
                 random_img = nprd.uniform(clip_min, clip_max, size=x.shape).astype(x.dtype)
+
+                if mask is not None:
+                    random_img = random_img * mask + x * (1 - mask)
+
                 random_class = np.argmax(
                     self.estimator.predict(np.array([random_img]), batch_size=self.batch_size), axis=1,
                 )[0]
@@ -271,6 +328,10 @@ class HopSkipJump(EvasionAttack):
             # The initial image unsatisfied
             for _ in range(self.init_size):
                 random_img = nprd.uniform(clip_min, clip_max, size=x.shape).astype(x.dtype)
+
+                if mask is not None:
+                    random_img = random_img * mask + x * (1 - mask)
+
                 random_class = np.argmax(
                     self.estimator.predict(np.array([random_img]), batch_size=self.batch_size), axis=1,
                 )[0]
@@ -296,7 +357,13 @@ class HopSkipJump(EvasionAttack):
         return initial_sample
 
     def _attack(
-        self, initial_sample: np.ndarray, original_sample: np.ndarray, target: int, clip_min: float, clip_max: float,
+        self,
+        initial_sample: np.ndarray,
+        original_sample: np.ndarray,
+        target: int,
+        mask: Optional[np.ndarray],
+        clip_min: float,
+        clip_max: float,
     ) -> np.ndarray:
         """
         Main function for the boundary attack.
@@ -304,6 +371,9 @@ class HopSkipJump(EvasionAttack):
         :param initial_sample: An initial adversarial example.
         :param original_sample: The original input.
         :param target: The target label.
+        :param mask: An array with a mask to be applied to the adversarial perturbations. Shape needs to be
+                     broadcastable to the shape of x. Any features for which the mask is zero will not be adversarially
+                     perturbed.
         :param clip_min: Minimum value of an example.
         :param clip_max: Maximum value of an example.
         :return: an adversarial example.
@@ -330,11 +400,13 @@ class HopSkipJump(EvasionAttack):
 
             # Next compute the number of evaluations and compute the update
             num_eval = min(int(self.init_eval * np.sqrt(self.curr_iter + 1)), self.max_eval)
+
             update = self._compute_update(
                 current_sample=current_sample,
                 num_eval=num_eval,
                 delta=delta,
                 target=target,
+                mask=mask,
                 clip_min=clip_min,
                 clip_max=clip_max,
             )
@@ -368,7 +440,7 @@ class HopSkipJump(EvasionAttack):
         current_sample: np.ndarray,
         original_sample: np.ndarray,
         target: int,
-        norm: int,
+        norm: Union[int, float, str],
         clip_min: float,
         clip_max: float,
         threshold: Optional[float] = None,
@@ -379,7 +451,7 @@ class HopSkipJump(EvasionAttack):
         :param current_sample: Current adversarial example.
         :param original_sample: The original input.
         :param target: The target label.
-        :param norm: Order of the norm. Possible values: np.inf or 2.
+        :param norm: Order of the norm. Possible values: "inf", np.inf or 2.
         :param clip_min: Minimum value of an example.
         :param clip_max: Maximum value of an example.
         :param threshold: The upper threshold in binary search.
@@ -449,7 +521,14 @@ class HopSkipJump(EvasionAttack):
         return delta
 
     def _compute_update(
-        self, current_sample: np.ndarray, num_eval: int, delta: float, target: int, clip_min: float, clip_max: float,
+        self,
+        current_sample: np.ndarray,
+        num_eval: int,
+        delta: float,
+        target: int,
+        mask: Optional[np.ndarray],
+        clip_min: float,
+        clip_max: float,
     ) -> np.ndarray:
         """
         Compute the update in Eq.(14).
@@ -458,6 +537,9 @@ class HopSkipJump(EvasionAttack):
         :param num_eval: The number of evaluations for estimating gradient.
         :param delta: The size of random perturbation.
         :param target: The target label.
+        :param mask: An array with a mask to be applied to the adversarial perturbations. Shape needs to be
+                     broadcastable to the shape of x. Any features for which the mask is zero will not be adversarially
+                     perturbed.
         :param clip_min: Minimum value of an example.
         :param clip_max: Maximum value of an example.
         :return: an updated perturbation.
@@ -469,6 +551,10 @@ class HopSkipJump(EvasionAttack):
         else:
             rnd_noise = np.random.uniform(low=-1, high=1, size=rnd_noise_shape).astype(ART_NUMPY_DTYPE)
 
+        # With mask
+        if mask is not None:
+            rnd_noise = rnd_noise * mask
+
         # Normalize random noise to fit into the range of input data
         rnd_noise = rnd_noise / np.sqrt(
             np.sum(rnd_noise ** 2, axis=tuple(range(len(rnd_noise_shape)))[1:], keepdims=True,)
@@ -523,14 +609,16 @@ class HopSkipJump(EvasionAttack):
         return result
 
     @staticmethod
-    def _interpolate(current_sample: np.ndarray, original_sample: np.ndarray, alpha: float, norm: int) -> np.ndarray:
+    def _interpolate(
+        current_sample: np.ndarray, original_sample: np.ndarray, alpha: float, norm: Union[int, float, str]
+    ) -> np.ndarray:
         """
         Interpolate a new sample based on the original and the current samples.
 
         :param current_sample: Current adversarial example.
         :param original_sample: The original input.
         :param alpha: The coefficient of interpolation.
-        :param norm: Order of the norm. Possible values: np.inf or 2.
+        :param norm: Order of the norm. Possible values: "inf", np.inf or 2.
         :return: An adversarial example.
         """
         if norm == 2:
@@ -542,8 +630,8 @@ class HopSkipJump(EvasionAttack):
 
     def _check_params(self) -> None:
         # Check if order of the norm is acceptable given current implementation
-        if self.norm not in [np.inf, int(2)]:
-            raise ValueError("Norm order must be either `np.inf` or 2.")
+        if self.norm not in [2, np.inf, "inf"]:
+            raise ValueError('Norm order must be either 2, `np.inf` or "inf".')
 
         if not isinstance(self.max_iter, (int, np.int)) or self.max_iter < 0:
             raise ValueError("The number of iterations must be a non-negative integer.")
@@ -559,3 +647,6 @@ class HopSkipJump(EvasionAttack):
 
         if not isinstance(self.init_size, (int, np.int)) or self.init_size <= 0:
             raise ValueError("The number of initial trials must be a positive integer.")
+
+        if not isinstance(self.verbose, bool):
+            raise ValueError("The argument `verbose` has to be of type bool.")
