@@ -26,18 +26,22 @@ al. for adversarial training.
 from __future__ import absolute_import, division, print_function, unicode_literals
 
 import logging
-from typing import Optional, TYPE_CHECKING
+from typing import Optional, Union, TYPE_CHECKING
 
 import numpy as np
+from tqdm import tqdm
 
 from art.config import ART_NUMPY_DTYPE
+from art.estimators.estimator import BaseEstimator, LossGradientsMixin
+from art.estimators.classification.classifier import ClassifierMixin
 from art.attacks.evasion.projected_gradient_descent.projected_gradient_descent_numpy import (
     ProjectedGradientDescentCommon,
 )
-from art.utils import compute_success, random_sphere
+from art.utils import compute_success, random_sphere, compute_success_array
 
 if TYPE_CHECKING:
     import torch
+    from art.estimators.classification.pytorch import PyTorchClassifier
 
 logger = logging.getLogger(__name__)
 
@@ -51,24 +55,26 @@ class ProjectedGradientDescentPyTorch(ProjectedGradientDescentCommon):
     | Paper link: https://arxiv.org/abs/1706.06083
     """
 
+    _estimator_requirements = (BaseEstimator, LossGradientsMixin, ClassifierMixin)
+
     def __init__(
         self,
-        estimator,
-        norm: int = np.inf,
-        eps: float = 0.3,
-        eps_step: float = 0.1,
+        estimator: Union["PyTorchClassifier"],
+        norm: Union[int, float, str] = np.inf,
+        eps: Union[int, float, np.ndarray] = 0.3,
+        eps_step: Union[int, float, np.ndarray] = 0.1,
         max_iter: int = 100,
         targeted: bool = False,
         num_random_init: int = 0,
         batch_size: int = 32,
         random_eps: bool = False,
+        verbose: bool = True,
     ):
         """
-        Create a :class:`.ProjectedGradientDescentPytorch` instance.
+        Create a :class:`.ProjectedGradientDescentPyTorch` instance.
 
         :param estimator: An trained estimator.
-        :type estimator: :class:`.BaseEstimator`
-        :param norm: The norm of the adversarial perturbation. Possible values: np.inf, 1 or 2.
+        :param norm: The norm of the adversarial perturbation. Possible values: "inf", np.inf, 1 or 2.
         :param eps: Maximum perturbation that the attacker can introduce.
         :param eps_step: Attack step size (input variation) at each iteration.
         :param random_eps: When True, epsilon is drawn randomly from truncated normal distribution. The literature
@@ -80,17 +86,11 @@ class ProjectedGradientDescentPyTorch(ProjectedGradientDescentCommon):
         :param num_random_init: Number of random initialisations within the epsilon ball. For num_random_init=0 starting
                                 at the original input.
         :param batch_size: Size of the batch on which adversarial samples are generated.
+        :param verbose: Show progress bars.
         """
-        if (
-            hasattr(estimator, "preprocessing")
-            and (estimator.preprocessing is not None and estimator.preprocessing != (0, 1))
-        ) or (
-            hasattr(estimator, "preprocessing_defences")
-            and (estimator.preprocessing_defences is not None and estimator.preprocessing_defences != [])
-        ):
+        if not estimator.all_framework_preprocessing:
             raise NotImplementedError(
-                "The framework-specific implementation currently does not apply preprocessing and "
-                "preprocessing defences."
+                "The framework-specific implementation only supports framework-specific preprocessing."
             )
 
         super(ProjectedGradientDescentPyTorch, self).__init__(
@@ -103,6 +103,7 @@ class ProjectedGradientDescentPyTorch(ProjectedGradientDescentCommon):
             num_random_init=num_random_init,
             batch_size=batch_size,
             random_eps=random_eps,
+            verbose=verbose,
         )
 
     def generate(self, x: np.ndarray, y: Optional[np.ndarray] = None, **kwargs) -> np.ndarray:
@@ -114,23 +115,25 @@ class ProjectedGradientDescentPyTorch(ProjectedGradientDescentCommon):
                   (nb_samples,). Only provide this parameter if you'd like to use true labels when crafting adversarial
                   samples. Otherwise, model predictions are used as labels to avoid the "label leaking" effect
                   (explained in this paper: https://arxiv.org/abs/1611.01236). Default is `None`.
-        :param mask: An array with a mask to be applied to the adversarial perturbations. Shape needs to be
-                     broadcastable to the shape of x. Any features for which the mask is zero will not be adversarially
-                     perturbed.
+        :param mask: An array with a mask broadcastable to input `x` defining where to apply adversarial perturbations.
+                     Shape needs to be broadcastable to the shape of x and can also be of the same shape as `x`. Any
+                     features for which the mask is zero will not be adversarially perturbed.
         :type mask: `np.ndarray`
         :return: An array holding the adversarial examples.
         """
         import torch  # lgtm [py/repeated-import]
 
+        mask = self._get_mask(x, **kwargs)
+
+        # Ensure eps is broadcastable
+        self._check_compatibility_input_and_eps(x=x)
+
         # Check whether random eps is enabled
         self._random_eps()
 
         # Set up targets
         targets = self._set_targets(x, y)
 
-        # Get the mask
-        mask = self._get_mask(x, **kwargs)
-
         # Create dataset
         if mask is not None:
             # Here we need to make a distinction: if the masks are different for each input, we need to index
@@ -159,42 +162,70 @@ class ProjectedGradientDescentPyTorch(ProjectedGradientDescentCommon):
         )
 
         # Start to compute adversarial examples
-        adv_x_best = None
-        rate_best = None
+        adv_x = x.astype(ART_NUMPY_DTYPE)
 
-        for _ in range(max(1, self.num_random_init)):
-            adv_x = x.astype(ART_NUMPY_DTYPE)
-
-            # Compute perturbation with batching
-            for (batch_id, batch_all) in enumerate(data_loader):
-                if mask is not None:
-                    (batch, batch_labels, mask_batch) = batch_all[0], batch_all[1], batch_all[2]
-                else:
-                    (batch, batch_labels, mask_batch) = batch_all[0], batch_all[1], None
-
-                batch_index_1, batch_index_2 = batch_id * self.batch_size, (batch_id + 1) * self.batch_size
-                adv_x[batch_index_1:batch_index_2] = self._generate_batch(batch, batch_labels, mask_batch)
-
-            if self.num_random_init > 1:
-                rate = 100 * compute_success(
-                    self.estimator, x, targets, adv_x, self.targeted, batch_size=self.batch_size
-                )
-                if rate_best is None or rate > rate_best or adv_x_best is None:
-                    rate_best = rate
-                    adv_x_best = adv_x
+        # Compute perturbation with batching
+        for (batch_id, batch_all) in enumerate(
+            tqdm(data_loader, desc="PGD - Batches", leave=False, disable=not self.verbose)
+        ):
+            if mask is not None:
+                (batch, batch_labels, mask_batch) = batch_all[0], batch_all[1], batch_all[2]
             else:
-                adv_x_best = adv_x
+                (batch, batch_labels, mask_batch) = batch_all[0], batch_all[1], None
+
+            batch_index_1, batch_index_2 = batch_id * self.batch_size, (batch_id + 1) * self.batch_size
+
+            # Compute batch_eps and batch_eps_step
+            if isinstance(self.eps, np.ndarray):
+                if len(self.eps.shape) == len(x.shape) and self.eps.shape[0] == x.shape[0]:
+                    batch_eps = self.eps[batch_index_1:batch_index_2]
+                    batch_eps_step = self.eps_step[batch_index_1:batch_index_2]
+
+                else:
+                    batch_eps = self.eps
+                    batch_eps_step = self.eps_step
+
+            else:
+                batch_eps = self.eps
+                batch_eps_step = self.eps_step
+
+            for rand_init_num in range(max(1, self.num_random_init)):
+                if rand_init_num == 0:
+                    # first iteration: use the adversarial examples as they are the only ones we have now
+                    adv_x[batch_index_1:batch_index_2] = self._generate_batch(
+                        x=batch, targets=batch_labels, mask=mask_batch, eps=batch_eps, eps_step=batch_eps_step
+                    )
+                else:
+                    adversarial_batch = self._generate_batch(
+                        x=batch, targets=batch_labels, mask=mask_batch, eps=batch_eps, eps_step=batch_eps_step
+                    )
+
+                    # return the successful adversarial examples
+                    attack_success = compute_success_array(
+                        self.estimator,
+                        batch,
+                        batch_labels,
+                        adversarial_batch,
+                        self.targeted,
+                        batch_size=self.batch_size,
+                    )
+                    adv_x[batch_index_1:batch_index_2][attack_success] = adversarial_batch[attack_success]
 
         logger.info(
             "Success rate of attack: %.2f%%",
-            rate_best
-            if rate_best is not None
-            else 100 * compute_success(self.estimator, x, y, adv_x_best, self.targeted, batch_size=self.batch_size),
+            100 * compute_success(self.estimator, x, y, adv_x, self.targeted, batch_size=self.batch_size),
         )
 
-        return adv_x_best
+        return adv_x
 
-    def _generate_batch(self, x: "torch.Tensor", targets: "torch.Tensor", mask: "torch.Tensor") -> np.ndarray:
+    def _generate_batch(
+        self,
+        x: "torch.Tensor",
+        targets: "torch.Tensor",
+        mask: "torch.Tensor",
+        eps: Union[int, float, np.ndarray],
+        eps_step: Union[int, float, np.ndarray],
+    ) -> np.ndarray:
         """
         Generate a batch of adversarial samples and return them in an array.
 
@@ -203,6 +234,8 @@ class ProjectedGradientDescentPyTorch(ProjectedGradientDescentCommon):
         :param mask: An array with a mask to be applied to the adversarial perturbations. Shape needs to be
                      broadcastable to the shape of x. Any features for which the mask is zero will not be adversarially
                      perturbed.
+        :param eps: Maximum perturbation that the attacker can introduce.
+        :param eps_step: Attack step size (input variation) at each iteration.
         :return: Adversarial examples.
         """
         inputs = x.to(self.estimator.device)
@@ -214,12 +247,14 @@ class ProjectedGradientDescentPyTorch(ProjectedGradientDescentCommon):
 
         for i_max_iter in range(self.max_iter):
             adv_x = self._compute_torch(
-                adv_x, inputs, targets, mask, self.eps, self.eps_step, self.num_random_init > 0 and i_max_iter == 0,
+                adv_x, inputs, targets, mask, eps, eps_step, self.num_random_init > 0 and i_max_iter == 0,
             )
 
         return adv_x.cpu().detach().numpy()
 
-    def _compute_perturbation(self, x: "torch.Tensor", y: "torch.Tensor", mask: "torch.Tensor") -> "torch.Tensor":
+    def _compute_perturbation(
+        self, x: "torch.Tensor", y: "torch.Tensor", mask: Optional["torch.Tensor"]
+    ) -> "torch.Tensor":
         """
         Compute perturbations.
 
@@ -228,9 +263,9 @@ class ProjectedGradientDescentPyTorch(ProjectedGradientDescentCommon):
                   (nb_samples,). Only provide this parameter if you'd like to use true labels when crafting adversarial
                   samples. Otherwise, model predictions are used as labels to avoid the "label leaking" effect
                   (explained in this paper: https://arxiv.org/abs/1611.01236). Default is `None`.
-        :param mask: An array with a mask to be applied to the adversarial perturbations. Shape needs to be
-                     broadcastable to the shape of x. Any features for which the mask is zero will not be adversarially
-                     perturbed.
+        :param mask: An array with a mask broadcastable to input `x` defining where to apply adversarial perturbations.
+                     Shape needs to be broadcastable to the shape of x and can also be of the same shape as `x`. Any
+                     features for which the mask is zero will not be adversarially perturbed.
         :return: Perturbations.
         """
         import torch  # lgtm [py/repeated-import]
@@ -239,10 +274,14 @@ class ProjectedGradientDescentPyTorch(ProjectedGradientDescentCommon):
         tol = 10e-8
 
         # Get gradient wrt loss; invert it if attack is targeted
-        grad = self.estimator.loss_gradient_framework(x, y) * (1 - 2 * int(self.targeted))
+        grad = self.estimator.loss_gradient(x=x, y=y) * (1 - 2 * int(self.targeted))
+
+        # Apply mask
+        if mask is not None:
+            grad = torch.where(mask == 0.0, torch.tensor(0.0), grad)
 
         # Apply norm bound
-        if self.norm == np.inf:
+        if self.norm in ["inf", np.inf]:
             grad = grad.sign()
 
         elif self.norm == 1:
@@ -255,12 +294,11 @@ class ProjectedGradientDescentPyTorch(ProjectedGradientDescentCommon):
 
         assert x.shape == grad.shape
 
-        if mask is None:
-            return grad
-        else:
-            return grad * mask
+        return grad
 
-    def _apply_perturbation(self, x: "torch.Tensor", perturbation: "torch.Tensor", eps_step: float) -> "torch.Tensor":
+    def _apply_perturbation(
+        self, x: "torch.Tensor", perturbation: "torch.Tensor", eps_step: Union[int, float, np.ndarray]
+    ) -> "torch.Tensor":
         """
         Apply perturbation on examples.
 
@@ -271,7 +309,8 @@ class ProjectedGradientDescentPyTorch(ProjectedGradientDescentCommon):
         """
         import torch  # lgtm [py/repeated-import]
 
-        x = x + eps_step * perturbation
+        eps_step = np.array(eps_step, dtype=ART_NUMPY_DTYPE)
+        x = x + torch.tensor(eps_step).to(self.estimator.device) * perturbation
 
         if self.estimator.clip_values is not None:
             clip_min, clip_max = self.estimator.clip_values
@@ -288,8 +327,8 @@ class ProjectedGradientDescentPyTorch(ProjectedGradientDescentCommon):
         x_init: "torch.Tensor",
         y: "torch.Tensor",
         mask: "torch.Tensor",
-        eps: float,
-        eps_step: float,
+        eps: Union[int, float, np.ndarray],
+        eps_step: Union[int, float, np.ndarray],
         random_init: bool,
     ) -> "torch.Tensor":
         """
@@ -301,9 +340,9 @@ class ProjectedGradientDescentPyTorch(ProjectedGradientDescentCommon):
                   (nb_samples,). Only provide this parameter if you'd like to use true labels when crafting adversarial
                   samples. Otherwise, model predictions are used as labels to avoid the "label leaking" effect
                   (explained in this paper: https://arxiv.org/abs/1611.01236).
-        :param mask: An array with a mask to be applied to the adversarial perturbations. Shape needs to be
-                     broadcastable to the shape of x. Any features for which the mask is zero will not be adversarially
-                     perturbed.
+        :param mask: An array with a mask broadcastable to input `x` defining where to apply adversarial perturbations.
+                     Shape needs to be broadcastable to the shape of x and can also be of the same shape as `x`. Any
+                     features for which the mask is zero will not be adversarially perturbed.
         :param eps: Maximum perturbation that the attacker can introduce.
         :param eps_step: Attack step size (input variation) at each iteration.
         :param random_init: Random initialisation within the epsilon ball. For random_init=False starting at the
@@ -314,7 +353,7 @@ class ProjectedGradientDescentPyTorch(ProjectedGradientDescentCommon):
 
         if random_init:
             n = x.shape[0]
-            m = np.prod(x.shape[1:])
+            m = np.prod(x.shape[1:]).item()
 
             random_perturbation = random_sphere(n, m, eps, self.norm).reshape(x.shape).astype(ART_NUMPY_DTYPE)
             random_perturbation = torch.from_numpy(random_perturbation).to(self.estimator.device)
@@ -348,13 +387,15 @@ class ProjectedGradientDescentPyTorch(ProjectedGradientDescentCommon):
 
         return x_adv
 
-    def _projection(self, values: "torch.Tensor", eps: float, norm_p: int) -> "torch.Tensor":
+    def _projection(
+        self, values: "torch.Tensor", eps: Union[int, float, np.ndarray], norm_p: Union[int, float, str]
+    ) -> "torch.Tensor":
         """
         Project `values` on the L_p norm ball of size `eps`.
 
         :param values: Values to clip.
         :param eps: Maximum norm allowed.
-        :param norm_p: L_p norm to use for clipping supporting 1, 2 and `np.Inf`.
+        :param norm_p: L_p norm to use for clipping supporting 1, 2, `np.Inf` and "inf".
         :return: Values of `values` after projection.
         """
         import torch  # lgtm [py/repeated-import]
@@ -364,18 +405,32 @@ class ProjectedGradientDescentPyTorch(ProjectedGradientDescentCommon):
         values_tmp = values.reshape(values.shape[0], -1)
 
         if norm_p == 2:
+            if isinstance(eps, np.ndarray):
+                raise NotImplementedError(
+                    "The parameter `eps` of type `np.ndarray` is not supported to use with norm 2."
+                )
+
             values_tmp = values_tmp * torch.min(
                 torch.tensor([1.0], dtype=torch.float32).to(self.estimator.device),
                 eps / (torch.norm(values_tmp, p=2, dim=1) + tol),
             ).unsqueeze_(-1)
 
         elif norm_p == 1:
+            if isinstance(eps, np.ndarray):
+                raise NotImplementedError(
+                    "The parameter `eps` of type `np.ndarray` is not supported to use with norm 1."
+                )
+
             values_tmp = values_tmp * torch.min(
                 torch.tensor([1.0], dtype=torch.float32).to(self.estimator.device),
                 eps / (torch.norm(values_tmp, p=1, dim=1) + tol),
             ).unsqueeze_(-1)
 
-        elif norm_p == np.inf:
+        elif norm_p in [np.inf, "inf"]:
+            if isinstance(eps, np.ndarray):
+                eps = eps * np.ones_like(values.cpu())
+                eps = eps.reshape([eps.shape[0], -1])
+
             values_tmp = values_tmp.sign() * torch.min(
                 values_tmp.abs(), torch.tensor([eps], dtype=torch.float32).to(self.estimator.device)
             )
