@@ -23,7 +23,7 @@ This module implements the Jacobian-based Saliency Map attack `SaliencyMapMethod
 from __future__ import absolute_import, division, print_function, unicode_literals
 
 import logging
-from typing import Optional
+from typing import Optional, Union, TYPE_CHECKING
 
 import numpy as np
 from tqdm import trange
@@ -31,12 +31,12 @@ from tqdm import trange
 from art.attacks.attack import EvasionAttack
 from art.config import ART_NUMPY_DTYPE
 from art.estimators.estimator import BaseEstimator
-from art.estimators.classification.classifier import (
-    ClassGradientsMixin,
-    ClassifierGradients,
-)
+from art.estimators.classification.classifier import ClassGradientsMixin
 from art.utils import check_and_transform_label_format, compute_success
 
+if TYPE_CHECKING:
+    from art.utils import CLASSIFIER_CLASS_LOSS_GRADIENTS_TYPE
+
 logger = logging.getLogger(__name__)
 
 
@@ -47,11 +47,16 @@ class SaliencyMapMethod(EvasionAttack):
     | Paper link: https://arxiv.org/abs/1511.07528
     """
 
-    attack_params = EvasionAttack.attack_params + ["theta", "gamma", "batch_size"]
+    attack_params = EvasionAttack.attack_params + ["theta", "gamma", "batch_size", "verbose"]
     _estimator_requirements = (BaseEstimator, ClassGradientsMixin)
 
     def __init__(
-        self, classifier: ClassifierGradients, theta: float = 0.1, gamma: float = 1.0, batch_size: int = 1,
+        self,
+        classifier: "CLASSIFIER_CLASS_LOSS_GRADIENTS_TYPE",
+        theta: float = 0.1,
+        gamma: float = 1.0,
+        batch_size: int = 1,
+        verbose: bool = True,
     ) -> None:
         """
         Create a SaliencyMapMethod instance.
@@ -60,11 +65,13 @@ class SaliencyMapMethod(EvasionAttack):
         :param theta: Amount of Perturbation introduced to each modified feature per step (can be positive or negative).
         :param gamma: Maximum fraction of features being perturbed (between 0 and 1).
         :param batch_size: Size of the batch on which adversarial samples are generated.
+        :param verbose: Show progress bars.
         """
-        super(SaliencyMapMethod, self).__init__(estimator=classifier)
+        super().__init__(estimator=classifier)
         self.theta = theta
         self.gamma = gamma
         self.batch_size = batch_size
+        self.verbose = verbose
         self._check_params()
 
     def generate(self, x: np.ndarray, y: Optional[np.ndarray] = None, **kwargs) -> np.ndarray:
@@ -94,7 +101,9 @@ class SaliencyMapMethod(EvasionAttack):
             targets = np.argmax(y, axis=1)
 
         # Compute perturbation with implicit batching
-        for batch_id in trange(int(np.ceil(x_adv.shape[0] / float(self.batch_size))), desc="JSMA"):
+        for batch_id in trange(
+            int(np.ceil(x_adv.shape[0] / float(self.batch_size))), desc="JSMA", disable=not self.verbose
+        ):
             batch_index_1, batch_index_2 = batch_id * self.batch_size, (batch_id + 1) * self.batch_size
             batch = x_adv[batch_index_1:batch_index_2]
 
@@ -175,14 +184,7 @@ class SaliencyMapMethod(EvasionAttack):
 
         return x_adv
 
-    def _check_params(self) -> None:
-        if self.gamma <= 0 or self.gamma > 1:
-            raise ValueError("The total perturbation percentage `gamma` must be between 0 and 1.")
-
-        if self.batch_size <= 0:
-            raise ValueError("The batch size `batch_size` has to be positive.")
-
-    def _saliency_map(self, x: np.ndarray, target: np.ndarray, search_space: np.ndarray) -> np.ndarray:
+    def _saliency_map(self, x: np.ndarray, target: Union[np.ndarray, int], search_space: np.ndarray) -> np.ndarray:
         """
         Compute the saliency map of `x`. Return the top 2 coefficients in `search_space` that maximize / minimize
         the saliency map.
@@ -206,3 +208,13 @@ class SaliencyMapMethod(EvasionAttack):
             ind = np.argpartition(-grads, -2, axis=1)[:, -2:]
 
         return ind
+
+    def _check_params(self) -> None:
+        if self.gamma <= 0 or self.gamma > 1:
+            raise ValueError("The total perturbation percentage `gamma` must be between 0 and 1.")
+
+        if self.batch_size <= 0:
+            raise ValueError("The batch size `batch_size` has to be positive.")
+
+        if not isinstance(self.verbose, bool):
+            raise ValueError("The argument `verbose` has to be of type bool.")
