@@ -24,16 +24,20 @@ import bisect
 import logging
 import math
 import random
-from typing import Union
+from typing import Optional, Union, TYPE_CHECKING
 
 import numpy as np
+from tqdm.auto import trange
 
 from art.config import ART_NUMPY_DTYPE
 from art.attacks.attack import EvasionAttack
 from art.estimators.estimator import BaseEstimator
-from art.estimators.classification.classifier import ClassifierMixin, ClassifierGradients
+from art.estimators.classification.classifier import ClassifierMixin
 from art.utils import check_and_transform_label_format, get_labels_np_array
 
+if TYPE_CHECKING:
+    from art.utils import CLASSIFIER_TYPE
+
 logger = logging.getLogger(__name__)
 
 
@@ -45,19 +49,35 @@ class SquareAttack(EvasionAttack):
         "eps",
         "p_init",
         "nb_restarts",
+        "batch_size",
+        "verbose",
     ]
 
     _estimator_requirements = (BaseEstimator, ClassifierMixin)
 
     def __init__(
         self,
-        estimator: ClassifierGradients,
-        norm: Union[float, int] = np.inf,
+        estimator: "CLASSIFIER_TYPE",
+        norm: Union[int, float, str] = np.inf,
         max_iter: int = 100,
         eps: float = 0.3,
         p_init: float = 0.8,
         nb_restarts: int = 1,
+        batch_size: int = 128,
+        verbose: bool = True,
     ):
+        """
+        Create a :class:`.SquareAttack` instance.
+
+        :param estimator: An trained estimator.
+        :param norm: The norm of the adversarial perturbation. Possible values: "inf", np.inf, 1 or 2.
+        :param max_iter: Maximum number of iterations.
+        :param eps: Maximum perturbation that the attacker can introduce.
+        :param p_init: Initial fraction of elements.
+        :param nb_restarts: Number of restarts.
+        :param batch_size: Batch size for estimator evaluations.
+        :param verbose: Show progress bars.
+        """
         super().__init__(estimator=estimator)
 
         self.norm = norm
@@ -65,10 +85,12 @@ class SquareAttack(EvasionAttack):
         self.eps = eps
         self.p_init = p_init
         self.nb_restarts = nb_restarts
+        self.batch_size = batch_size
+        self.verbose = verbose
         self._check_params()
 
     def _get_logits_diff(self, x: np.ndarray, y: np.ndarray) -> np.ndarray:
-        y_pred = self.estimator.predict(x)
+        y_pred = self.estimator.predict(x, batch_size=self.batch_size)
 
         logit_correct = np.take_along_axis(y_pred, np.expand_dims(np.argmax(y, axis=1), axis=1), axis=1)
         logit_highest_incorrect = np.take_along_axis(
@@ -85,19 +107,16 @@ class SquareAttack(EvasionAttack):
 
         return self.p_init * p_ratio[i_ratio]
 
-    def generate(self, x, y=None, **kwargs):
+    def generate(self, x: np.ndarray, y: Optional[np.ndarray] = None, **kwargs) -> np.ndarray:
         """
         Generate adversarial samples and return them in an array.
 
         :param x: An array with the original inputs.
-        :type x: `np.ndarray`
         :param y: Target values (class labels) one-hot-encoded of shape `(nb_samples, nb_classes)` or indices of shape
                   (nb_samples,). Only provide this parameter if you'd like to use true labels when crafting adversarial
                   samples. Otherwise, model predictions are used as labels to avoid the "label leaking" effect
                   (explained in this paper: https://arxiv.org/abs/1611.01236). Default is `None`.
-        :type y: `np.ndarray`
         :return: An array holding the adversarial examples.
-        :rtype: `np.ndarray`
         """
         if x.ndim != 4:
             raise ValueError("Unrecognized input dimension. Attack can only be applied to image data.")
@@ -120,10 +139,10 @@ class SquareAttack(EvasionAttack):
             width = x.shape[2]
             channels = x.shape[3]
 
-        for i_restart in range(self.nb_restarts):
+        for _ in trange(self.nb_restarts, desc="SquareAttack - restarts", disable=not self.verbose):
 
             # Determine correctly predicted samples
-            y_pred = self.estimator.predict(x_adv)
+            y_pred = self.estimator.predict(x_adv, batch_size=self.batch_size)
             sample_is_robust = np.argmax(y_pred, axis=1) == np.argmax(y, axis=1)
 
             if np.sum(sample_is_robust) == 0:
@@ -134,7 +153,7 @@ class SquareAttack(EvasionAttack):
             y_robust = y[sample_is_robust]
             sample_logits_diff_init = self._get_logits_diff(x_robust, y_robust)
 
-            if self.norm == np.inf:
+            if self.norm in [np.inf, "inf"]:
 
                 if self.estimator.channels_first:
                     size = (x_robust.shape[0], channels, 1, width)
@@ -155,12 +174,14 @@ class SquareAttack(EvasionAttack):
 
                 x_adv[sample_is_robust] = x_robust
 
-                for i_iter in range(self.max_iter):
+                for i_iter in trange(
+                    self.max_iter, desc="SquareAttack - iterations", leave=False, disable=not self.verbose
+                ):
 
                     percentage_of_elements = self._get_percentage_of_elements(i_iter)
 
                     # Determine correctly predicted samples
-                    y_pred = self.estimator.predict(x_adv)
+                    y_pred = self.estimator.predict(x_adv, batch_size=self.batch_size)
                     sample_is_robust = np.argmax(y_pred, axis=1) == np.argmax(y, axis=1)
 
                     if np.sum(sample_is_robust) == 0:
@@ -279,12 +300,14 @@ class SquareAttack(EvasionAttack):
 
                 x_adv[sample_is_robust] = x_robust
 
-                for i_iter in range(self.max_iter):
+                for i_iter in trange(
+                    self.max_iter, desc="SquareAttack - iterations", leave=False, disable=not self.verbose
+                ):
 
                     percentage_of_elements = self._get_percentage_of_elements(i_iter)
 
                     # Determine correctly predicted samples
-                    y_pred = self.estimator.predict(x_adv)
+                    y_pred = self.estimator.predict(x_adv, batch_size=self.batch_size)
                     sample_is_robust = np.argmax(y_pred, axis=1) == np.argmax(y, axis=1)
 
                     if np.sum(sample_is_robust) == 0:
@@ -312,7 +335,7 @@ class SquareAttack(EvasionAttack):
                         new_deltas_mask[
                             :, :, height_start : height_start + height_tile, width_start : width_start + height_tile
                         ] = 1.0
-                        W_1_norm = np.sqrt(
+                        w_1_norm = np.sqrt(
                             np.sum(
                                 delta_x_robust_init[
                                     :,
@@ -329,7 +352,7 @@ class SquareAttack(EvasionAttack):
                         new_deltas_mask[
                             :, height_start : height_start + height_tile, width_start : width_start + height_tile, :
                         ] = 1.0
-                        W_1_norm = np.sqrt(
+                        w_1_norm = np.sqrt(
                             np.sum(
                                 delta_x_robust_init[
                                     :,
@@ -363,7 +386,7 @@ class SquareAttack(EvasionAttack):
                         ] = 1.0
 
                     norms_x_robust = np.sqrt(np.sum((x_robust - x_init) ** 2, axis=(1, 2, 3), keepdims=True))
-                    W_norm = np.sqrt(
+                    w_norm = np.sqrt(
                         np.sum(
                             (delta_x_robust_init * np.maximum(new_deltas_mask, new_deltas_mask_2)) ** 2,
                             axis=(1, 2, 3),
@@ -389,18 +412,18 @@ class SquareAttack(EvasionAttack):
                     if self.estimator.channels_first:
                         delta_new += delta_x_robust_init[
                             :, :, height_start : height_start + height_tile, width_start : width_start + height_tile
-                        ] / (np.maximum(1e-9, W_1_norm))
+                        ] / (np.maximum(1e-9, w_1_norm))
                     else:
                         delta_new += delta_x_robust_init[
                             :, height_start : height_start + height_tile, width_start : width_start + height_tile, :
-                        ] / (np.maximum(1e-9, W_1_norm))
+                        ] / (np.maximum(1e-9, w_1_norm))
 
                     diff_norm = (self.eps * np.ones(delta_new.shape)) ** 2 - norms_x_robust ** 2
                     diff_norm[diff_norm < 0.0] = 0.0
 
                     if self.estimator.channels_first:
                         delta_new /= np.sqrt(np.sum(delta_new ** 2, axis=(2, 3), keepdims=True)) * np.sqrt(
-                            diff_norm / channels + W_norm ** 2
+                            diff_norm / channels + w_norm ** 2
                         )
                         delta_x_robust_init[
                             :,
@@ -413,7 +436,7 @@ class SquareAttack(EvasionAttack):
                         ] = delta_new
                     else:
                         delta_new /= np.sqrt(np.sum(delta_new ** 2, axis=(1, 2), keepdims=True)) * np.sqrt(
-                            diff_norm / channels + W_norm ** 2
+                            diff_norm / channels + w_norm ** 2
                         )
                         delta_x_robust_init[
                             :,
@@ -444,8 +467,8 @@ class SquareAttack(EvasionAttack):
         return x_adv
 
     def _check_params(self) -> None:
-        if self.norm not in [1, 2, np.inf]:
-            raise ValueError("The argument norm has to be either 1, 2, or np.inf.")
+        if self.norm not in [1, 2, np.inf, "inf"]:
+            raise ValueError('The argument norm has to be either 1, 2, np.inf, or "inf".')
 
         if not isinstance(self.max_iter, int) or self.max_iter <= 0:
             raise ValueError("The argument max_iter has to be of type int and larger than zero.")
@@ -458,3 +481,9 @@ class SquareAttack(EvasionAttack):
 
         if not isinstance(self.nb_restarts, int) or self.nb_restarts <= 0:
             raise ValueError("The argument nb_restarts has to be of type int and larger than zero.")
+
+        if not isinstance(self.batch_size, int) or self.batch_size <= 0:
+            raise ValueError("The argument batch_size has to be of type int and larger than zero.")
+
+        if not isinstance(self.verbose, bool):
+            raise ValueError("The argument `verbose` has to be of type bool.")
