@@ -23,21 +23,20 @@ This module implements the virtual adversarial attack. It was originally used fo
 from __future__ import absolute_import, division, print_function, unicode_literals
 
 import logging
-from typing import Optional, Union
+from typing import Optional, TYPE_CHECKING
 
 import numpy as np
 from tqdm import trange
 
 from art.attacks.attack import EvasionAttack
 from art.config import ART_NUMPY_DTYPE
-from art.estimators.estimator import BaseEstimator, NeuralNetworkMixin
-from art.estimators.classification.classifier import (
-    ClassGradientsMixin,
-    ClassifierGradients,
-    ClassifierNeuralNetwork,
-)
+from art.estimators.estimator import BaseEstimator
+from art.estimators.classification.classifier import ClassifierMixin
 from art.utils import compute_success
 
+if TYPE_CHECKING:
+    from art.utils import CLASSIFIER_TYPE
+
 logger = logging.getLogger(__name__)
 
 
@@ -53,16 +52,18 @@ class VirtualAdversarialMethod(EvasionAttack):
         "finite_diff",
         "max_iter",
         "batch_size",
+        "verbose",
     ]
-    _estimator_requirements = (BaseEstimator, NeuralNetworkMixin, ClassGradientsMixin)
+    _estimator_requirements = (BaseEstimator, ClassifierMixin)
 
     def __init__(
         self,
-        classifier: Union[ClassifierGradients, ClassifierNeuralNetwork],
+        classifier: "CLASSIFIER_TYPE",
         max_iter: int = 10,
         finite_diff: float = 1e-6,
         eps: float = 0.1,
         batch_size: int = 1,
+        verbose: bool = True,
     ) -> None:
         """
         Create a :class:`.VirtualAdversarialMethod` instance.
@@ -72,12 +73,14 @@ class VirtualAdversarialMethod(EvasionAttack):
         :param finite_diff: The finite difference parameter.
         :param max_iter: The maximum number of iterations.
         :param batch_size: Size of the batch on which adversarial samples are generated.
+        :param verbose: Show progress bars.
         """
-        super(VirtualAdversarialMethod, self).__init__(classifier)
+        super().__init__(estimator=classifier)
         self.finite_diff = finite_diff
         self.eps = eps
         self.max_iter = max_iter
         self.batch_size = batch_size
+        self.verbose = verbose
         self._check_params()
 
     def generate(self, x: np.ndarray, y: Optional[np.ndarray] = None, **kwargs) -> np.ndarray:
@@ -99,7 +102,9 @@ class VirtualAdversarialMethod(EvasionAttack):
         preds_rescaled = preds
 
         # Compute perturbation with implicit batching
-        for batch_id in trange(int(np.ceil(x_adv.shape[0] / float(self.batch_size))), desc="VAT"):
+        for batch_id in trange(
+            int(np.ceil(x_adv.shape[0] / float(self.batch_size))), desc="VAT", disable=not self.verbose
+        ):
             batch_index_1, batch_index_2 = batch_id * self.batch_size, (batch_id + 1) * self.batch_size
             batch = x_adv[batch_index_1:batch_index_2]
             batch = batch.reshape((batch.shape[0], -1))
@@ -205,3 +210,6 @@ class VirtualAdversarialMethod(EvasionAttack):
 
         if self.batch_size <= 0:
             raise ValueError("The batch size `batch_size` has to be positive.")
+
+        if not isinstance(self.verbose, bool):
+            raise ValueError("The argument `verbose` has to be of type bool.")
