@@ -53,18 +53,24 @@ class PoisoningAttackBackdoor(PoisoningAttackBlackBox):
         self.perturbation = perturbation
         self._check_params()
 
-    def poison(self, x: np.ndarray, y: Optional[np.ndarray] = None, **kwargs) -> Tuple[np.ndarray, np.ndarray]:
+    def poison(
+        self, x: np.ndarray, y: Optional[np.ndarray] = None, broadcast=False, **kwargs
+    ) -> Tuple[np.ndarray, np.ndarray]:
         """
-        Iteratively finds optimal attack points starting at values at x.
+        Calls perturbation function on input x and returns the perturbed input and poison labels for the data.
 
         :param x: An array with the points that initialize attack points.
         :param y: The target labels for the attack.
+        :param broadcast: whether or not to broadcast single target label
         :return: An tuple holding the `(poisoning_examples, poisoning_labels)`.
         """
         if y is None:
             raise ValueError("Target labels `y` need to be provided for a targeted attack.")
         else:
-            y_attack = np.copy(y)
+            if broadcast:
+                y_attack = np.broadcast_to(y, (x.shape[0], y.shape[0]))
+            else:
+                y_attack = np.copy(y)
 
         num_poison = len(x)
         if num_poison == 0:
