@@ -21,7 +21,6 @@ This module loads and provides configuration parameters for ART.
 import json
 import logging
 import os
-from typing import Tuple, Union, Optional
 
 import numpy as np
 
@@ -30,9 +29,7 @@ logger = logging.getLogger(__name__)
 # ------------------------------------------------------------------------------------------------- CONSTANTS AND TYPES
 
 ART_NUMPY_DTYPE = np.float32
-DATASET_TYPE = Tuple[Tuple[np.ndarray, np.ndarray], Tuple[np.ndarray, np.ndarray], float, float]
-CLIP_VALUES_TYPE = Tuple[Union[int, float, np.ndarray], Union[int, float, np.ndarray]]
-PREPROCESSING_TYPE = Optional[Tuple[Union[int, float, np.ndarray], Union[int, float, np.ndarray]]]
+ART_DATA_PATH: str
 
 # --------------------------------------------------------------------------------------------- DEFAULT PACKAGE CONFIGS
 
@@ -41,6 +38,23 @@ if not os.access(_folder, os.W_OK):
     _folder = "/tmp"
 _folder = os.path.join(_folder, ".art")
 
+
+def set_data_path(path):
+    """
+    Set the path for ART's data directory (ART_DATA_PATH).
+    """
+    expanded_path = os.path.abspath(os.path.expanduser(path))
+    os.makedirs(expanded_path, exist_ok=True)
+    if not os.access(expanded_path, os.R_OK):
+        raise OSError(f"path {expanded_path} cannot be read from")
+    if not os.access(expanded_path, os.W_OK):
+        logger.warning(f"path %s is read only", expanded_path)
+
+    global ART_DATA_PATH
+    ART_DATA_PATH = expanded_path
+    logger.info(f"set ART_DATA_PATH to %s", expanded_path)
+
+
 # Load data from configuration file if it exists. Otherwise create one.
 _config_path = os.path.expanduser(os.path.join(_folder, "config.json"))
 if os.path.exists(_config_path):
@@ -76,11 +90,5 @@ if not os.path.exists(_config_path):
     except IOError:
         logger.warning("Unable to create configuration file", exc_info=True)
 
-if not os.path.exists(_config["ART_DATA_PATH"]):
-    try:
-        os.makedirs(_config["ART_DATA_PATH"])
-    except OSError:
-        logger.warning("Unable to create folder for ART_DATA_PATH dir.", exc_info=True)
-
 if "ART_DATA_PATH" in _config:
-    ART_DATA_PATH = _config["ART_DATA_PATH"]
+    set_data_path(_config["ART_DATA_PATH"])
