@@ -47,22 +47,11 @@ class ReverseSigmoid(Postprocessor):
         :param apply_fit: True if applied during fitting/training.
         :param apply_predict: True if applied during predicting.
         """
-        super(ReverseSigmoid, self).__init__()
-        self._is_fitted = True
-        self._apply_fit = apply_fit
-        self._apply_predict = apply_predict
+        super().__init__(is_fitted=True, apply_fit=apply_fit, apply_predict=apply_predict)
         self.beta = beta
         self.gamma = gamma
         self._check_params()
 
-    @property
-    def apply_fit(self) -> bool:
-        return self._apply_fit
-
-    @property
-    def apply_predict(self) -> bool:
-        return self._apply_predict
-
     def __call__(self, preds: np.ndarray) -> np.ndarray:
         """
         Perform model postprocessing and return postprocessed output.
@@ -109,12 +98,6 @@ class ReverseSigmoid(Postprocessor):
 
         return reverse_sigmoid
 
-    def fit(self, preds: np.ndarray, **kwargs) -> None:
-        """
-        No parameters to learn for this method; do nothing.
-        """
-        pass
-
     def _check_params(self) -> None:
         if self.beta <= 0:
             raise ValueError("Magnitude parameter must be positive.")
