@@ -42,21 +42,10 @@ class Rounded(Postprocessor):
         :param apply_fit: True if applied during fitting/training.
         :param apply_predict: True if applied during predicting.
         """
-        super(Rounded, self).__init__()
-        self._is_fitted = True
-        self._apply_fit = apply_fit
-        self._apply_predict = apply_predict
+        super().__init__(is_fitted=True, apply_fit=apply_fit, apply_predict=apply_predict)
         self.decimals = decimals
         self._check_params()
 
-    @property
-    def apply_fit(self) -> bool:
-        return self._apply_fit
-
-    @property
-    def apply_predict(self) -> bool:
-        return self._apply_predict
-
     def __call__(self, preds: np.ndarray) -> np.ndarray:
         """
         Perform model postprocessing and return postprocessed output.
@@ -66,12 +55,6 @@ class Rounded(Postprocessor):
         """
         return np.around(preds, decimals=self.decimals)
 
-    def fit(self, preds: np.ndarray, **kwargs) -> None:
-        """
-        No parameters to learn for this method; do nothing.
-        """
-        pass
-
     def _check_params(self) -> None:
         if not isinstance(self.decimals, (int, np.int)) or self.decimals <= 0:
             raise ValueError("Number of decimal places must be a positive integer.")
