@@ -33,11 +33,12 @@ from typing import Optional, Tuple, TYPE_CHECKING
 import numpy as np
 from tqdm import tqdm
 
-from art.config import ART_NUMPY_DTYPE, CLIP_VALUES_TYPE
+from art.config import ART_NUMPY_DTYPE
 from art.defences.preprocessor.preprocessor import Preprocessor
 
 if TYPE_CHECKING:
-    from art.estimators.classification.classifier import ClassifierNeuralNetwork
+    from art.utils import CLIP_VALUES_TYPE, CLASSIFIER_NEURALNETWORK_TYPE
+
 
 logger = logging.getLogger(__name__)
 
@@ -53,16 +54,17 @@ class PixelDefend(Preprocessor):
         https://arxiv.org/abs/1902.06705
     """
 
-    params = ["clip_values", "eps", "pixel_cnn"]
+    params = ["clip_values", "eps", "pixel_cnn", "verbose"]
 
     def __init__(
         self,
-        clip_values: CLIP_VALUES_TYPE = (0.0, 1.0),
+        clip_values: "CLIP_VALUES_TYPE" = (0.0, 1.0),
         eps: int = 16,
-        pixel_cnn: Optional["ClassifierNeuralNetwork"] = None,
+        pixel_cnn: Optional["CLASSIFIER_NEURALNETWORK_TYPE"] = None,
         batch_size: int = 128,
         apply_fit: bool = False,
         apply_predict: bool = True,
+        verbose: bool = False,
     ) -> None:
         """
         Create an instance of pixel defence.
@@ -71,25 +73,16 @@ class PixelDefend(Preprocessor):
                for features.
         :param eps: Defense parameter 0-255.
         :param pixel_cnn: Pre-trained PixelCNN model.
+        :param verbose: Show progress bars.
         """
-        super(PixelDefend, self).__init__()
-        self._is_fitted = True
-        self._apply_fit = apply_fit
-        self._apply_predict = apply_predict
+        super().__init__(is_fitted=True, apply_fit=apply_fit, apply_predict=apply_predict)
         self.clip_values = clip_values
         self.eps = eps
         self.batch_size = batch_size
         self.pixel_cnn = pixel_cnn
+        self.verbose = verbose
         self._check_params()
 
-    @property
-    def apply_fit(self) -> bool:
-        return self._apply_fit
-
-    @property
-    def apply_predict(self) -> bool:
-        return self._apply_predict
-
     def __call__(self, x: np.ndarray, y: Optional[np.ndarray] = None) -> Tuple[np.ndarray, Optional[np.ndarray]]:
         """
         Apply pixel defence to sample `x`.
@@ -113,7 +106,7 @@ class PixelDefend(Preprocessor):
         x = x.reshape((x.shape[0], -1))
 
         # Start defence one image at a time
-        for i, x_i in enumerate(tqdm(x, desc="PixelDefend")):
+        for i, x_i in enumerate(tqdm(x, desc="PixelDefend", disable=not self.verbose)):
             for feat_index in range(x.shape[1]):
                 # Setup the search space
                 f_probs = probs[i, feat_index, :]
@@ -142,22 +135,17 @@ class PixelDefend(Preprocessor):
 
         return x, y
 
-    def estimate_gradient(self, x: np.ndarray, grad: np.ndarray) -> np.ndarray:
-        raise grad
-
-    def fit(self, x: np.ndarray, y: Optional[np.ndarray] = None, **kwargs) -> None:
-        """
-        No parameters to learn for this method; do nothing.
-        """
-        pass
-
     def _check_params(self) -> None:
-        from art.estimators.classification.classifier import ClassifierMixin
 
         if not isinstance(self.eps, (int, np.int)) or self.eps < 0 or self.eps > 255:
             raise ValueError("The defense parameter must be between 0 and 255.")
 
-        if hasattr(self, "pixel_cnn") and not isinstance(self.pixel_cnn, ClassifierMixin):
+        from art.estimators.classification.classifier import ClassifierMixin
+        from art.estimators.estimator import NeuralNetworkMixin
+
+        if hasattr(self, "pixel_cnn") and not (
+            isinstance(self.pixel_cnn, ClassifierMixin) and isinstance(self.pixel_cnn, NeuralNetworkMixin)
+        ):
             raise TypeError("PixelCNN model must be of type Classifier.")
 
         if np.array(self.clip_values[0] >= self.clip_values[1]).any():
@@ -171,3 +159,6 @@ class PixelDefend(Preprocessor):
 
         if self.batch_size <= 0:
             raise ValueError("The batch size `batch_size` has to be positive.")
+
+        if not isinstance(self.verbose, bool):
+            raise ValueError("The argument `verbose` has to be of type bool.")
