@@ -30,7 +30,7 @@ from typing import Any, Dict, List, Optional, Tuple, Union, TYPE_CHECKING
 import numpy as np
 import six
 
-from art.config import ART_DATA_PATH, CLIP_VALUES_TYPE, PREPROCESSING_TYPE
+from art import config
 from art.estimators.classification.classifier import (
     ClassGradientsMixin,
     ClassifierMixin,
@@ -39,8 +39,10 @@ from art.estimators.pytorch import PyTorchEstimator
 from art.utils import Deprecated, deprecated_keyword_arg, check_and_transform_label_format
 
 if TYPE_CHECKING:
+    # pylint: disable=C0412
     import torch
 
+    from art.utils import CLIP_VALUES_TYPE, PREPROCESSING_TYPE
     from art.data_generators import DataGenerator
     from art.defences.preprocessor import Preprocessor
     from art.defences.postprocessor import Postprocessor
@@ -53,7 +55,7 @@ class PyTorchClassifier(ClassGradientsMixin, ClassifierMixin, PyTorchEstimator):
     This class implements a classifier with the PyTorch framework.
     """
 
-    @deprecated_keyword_arg("channel_index", end_version="1.5.0", replaced_by="channels_first")
+    @deprecated_keyword_arg("channel_index", end_version="1.6.0", replaced_by="channels_first")
     def __init__(
         self,
         model: "torch.nn.Module",
@@ -61,12 +63,15 @@ class PyTorchClassifier(ClassGradientsMixin, ClassifierMixin, PyTorchEstimator):
         input_shape: Tuple[int, ...],
         nb_classes: int,
         optimizer: Optional["torch.optim.Optimizer"] = None,  # type: ignore
+        use_amp: bool = False,
+        opt_level: str = "O1",
+        loss_scale: Optional[Union[float, str]] = "dynamic",
         channel_index=Deprecated,
         channels_first: bool = True,
-        clip_values: Optional[CLIP_VALUES_TYPE] = None,
+        clip_values: Optional["CLIP_VALUES_TYPE"] = None,
         preprocessing_defences: Union["Preprocessor", List["Preprocessor"], None] = None,
         postprocessing_defences: Union["Postprocessor", List["Postprocessor"], None] = None,
-        preprocessing: PREPROCESSING_TYPE = (0, 1),
+        preprocessing: "PREPROCESSING_TYPE" = (0, 1),
         device_type: str = "gpu",
     ) -> None:
         """
@@ -78,6 +83,13 @@ class PyTorchClassifier(ClassGradientsMixin, ClassifierMixin, PyTorchEstimator):
                categorical, i.e. not converted to one-hot encoding.
         :param input_shape: The shape of one input instance.
         :param optimizer: The optimizer used to train the classifier.
+        :param use_amp: Whether to use the automatic mixed precision tool to enable mixed precision training or
+                        gradient computation, e.g. with loss gradient computation. When set to True, this option is
+                        only triggered if there are GPUs available.
+        :param opt_level: Specify a pure or mixed precision optimization level. Used when use_amp is True. Accepted
+                          values are `O0`, `O1`, `O2`, and `O3`.
+        :param loss_scale: Loss scaling. Used when use_amp is True. If passed as a string, must be a string
+                           representing a number, e.g., “1.0”, or the string “dynamic”.
         :param nb_classes: The number of classes of the model.
         :param optimizer: The optimizer used to train the classifier.
         :param channel_index: Index of the axis in data containing the color channels or features.
@@ -89,14 +101,14 @@ class PyTorchClassifier(ClassGradientsMixin, ClassifierMixin, PyTorchEstimator):
                the shape of clip values needs to match the total number of features.
         :param preprocessing_defences: Preprocessing defence(s) to be applied by the classifier.
         :param postprocessing_defences: Postprocessing defence(s) to be applied by the classifier.
-        :param preprocessing: Tuple of the form `(subtractor, divider)` of floats or `np.ndarray` of values to be
+        :param preprocessing: Tuple of the form `(subtrahend, divisor)` of floats or `np.ndarray` of values to be
                used for data preprocessing. The first value will be subtracted from the input. The input will then
                be divided by the second one.
         :param device_type: Type of device on which the classifier is run, either `gpu` or `cpu`.
         """
         import torch  # lgtm [py/repeated-import]
 
-        # Remove in 1.5.0
+        # Remove in 1.6.0
         if channel_index == 3:
             channels_first = False
         elif channel_index == 1:
@@ -105,40 +117,67 @@ class PyTorchClassifier(ClassGradientsMixin, ClassifierMixin, PyTorchEstimator):
             raise ValueError("Not a proper channel_index. Use channels_first.")
 
         super().__init__(
+            model=model,
             clip_values=clip_values,
             channel_index=channel_index,
             channels_first=channels_first,
             preprocessing_defences=preprocessing_defences,
             postprocessing_defences=postprocessing_defences,
             preprocessing=preprocessing,
+            device_type=device_type,
         )
         self._nb_classes = nb_classes
         self._input_shape = input_shape
         self._model = self._make_model_wrapper(model)
         self._loss = loss
         self._optimizer = optimizer
+        self._use_amp = use_amp
         self._learning_phase: Optional[bool] = None
 
         # Get the internal layers
         self._layer_names = self._model.get_layers
 
-        # Set device
-        if device_type == "cpu" or not torch.cuda.is_available():
-            self._device = torch.device("cpu")
-        else:
-            cuda_idx = torch.cuda.current_device()
-            self._device = torch.device("cuda:{}".format(cuda_idx))
-
         self._model.to(self._device)
 
         # Index of layer at which the class gradients should be calculated
         self._layer_idx_gradients = -1
 
-        # Check if the loss function requires as input index labels instead of one-hot-encoded labels
-        if isinstance(loss, (torch.nn.CrossEntropyLoss, torch.nn.NLLLoss, torch.nn.MultiMarginLoss),):
+        if isinstance(self._loss, (torch.nn.CrossEntropyLoss, torch.nn.NLLLoss, torch.nn.MultiMarginLoss),):
             self._reduce_labels = True
+            self._int_labels = True
+        elif isinstance(self._loss, (torch.nn.BCELoss),):
+            self._reduce_labels = True
+            self._int_labels = False
         else:
             self._reduce_labels = False
+            self._int_labels = False
+
+        # Setup for AMP use
+        if self._use_amp:
+            from apex import amp
+
+            if self._optimizer is None:
+                logger.warning(
+                    "An optimizer is needed to use the automatic mixed precision tool, but none for provided. "
+                    "A default optimizer is used."
+                )
+
+                # Create the optimizers
+                parameters = self._model.parameters()
+                self._optimizer = torch.optim.SGD(parameters, lr=0.01)
+
+            if self.device.type == "cpu":
+                enabled = False
+            else:
+                enabled = True
+
+            self._model, self._optimizer = amp.initialize(
+                models=self._model,
+                optimizers=self._optimizer,
+                enabled=enabled,
+                opt_level=opt_level,
+                loss_scale=loss_scale,
+            )
 
     @property
     def device(self) -> "torch.device":
@@ -153,6 +192,34 @@ class PyTorchClassifier(ClassGradientsMixin, ClassifierMixin, PyTorchEstimator):
     def model(self) -> "torch.nn.Module":
         return self._model._model
 
+    @property
+    def input_shape(self) -> Tuple[int, ...]:
+        """
+        Return the shape of one input sample.
+
+        :return: Shape of one input sample.
+        """
+        return self._input_shape  # type: ignore
+
+    def reduce_labels(self, y: Union[np.ndarray, "torch.Tensor"]) -> Union[np.ndarray, "torch.Tensor"]:
+        import torch  # lgtm [py/repeated-import]
+
+        # Check if the loss function requires as input index labels instead of one-hot-encoded labels
+        if self._reduce_labels and self._int_labels:
+            if isinstance(y, torch.Tensor):
+                return torch.argmax(y, dim=1)
+            else:
+                return np.argmax(y, axis=1)
+        elif self._reduce_labels:  # float labels
+            if isinstance(y, torch.Tensor):
+                return torch.argmax(y, dim=1).type("torch.FloatTensor")
+            else:
+                y_index = np.argmax(y, axis=1).astype(np.float32)
+                y_index = np.expand_dims(y_index, axis=1)
+                return y_index
+        else:
+            return y
+
     def predict(self, x: np.ndarray, batch_size: int = 128, **kwargs) -> np.ndarray:
         """
         Perform prediction for a batch of inputs.
@@ -163,6 +230,7 @@ class PyTorchClassifier(ClassGradientsMixin, ClassifierMixin, PyTorchEstimator):
         """
         import torch  # lgtm [py/repeated-import]
 
+        # Put the model in the eval mode
         self._model.eval()
 
         # Apply preprocessing
@@ -202,6 +270,9 @@ class PyTorchClassifier(ClassGradientsMixin, ClassifierMixin, PyTorchEstimator):
         """
         import torch  # lgtm [py/repeated-import]
 
+        # Put the model in the training mode
+        self._model.train()
+
         if self._optimizer is None:
             raise ValueError("An optimizer is needed to train the model, but none for provided.")
 
@@ -211,8 +282,7 @@ class PyTorchClassifier(ClassGradientsMixin, ClassifierMixin, PyTorchEstimator):
         x_preprocessed, y_preprocessed = self._apply_preprocessing(x, y, fit=True)
 
         # Check label shape
-        if self._reduce_labels:
-            y_preprocessed = np.argmax(y_preprocessed, axis=1)
+        y_preprocessed = self.reduce_labels(y_preprocessed)
 
         num_batch = int(np.ceil(len(x_preprocessed) / float(batch_size)))
         ind = np.arange(len(x_preprocessed))
@@ -236,8 +306,16 @@ class PyTorchClassifier(ClassGradientsMixin, ClassifierMixin, PyTorchEstimator):
                 # Form the loss function
                 loss = self._loss(model_outputs[-1], o_batch)
 
-                # Actual training
-                loss.backward()
+                # Do training
+                if self._use_amp:
+                    from apex import amp
+
+                    with amp.scale_loss(loss, self._optimizer) as scaled_loss:
+                        scaled_loss.backward()
+
+                else:
+                    loss.backward()
+
                 self._optimizer.step()
 
     def fit_generator(self, generator: "DataGenerator", nb_epochs: int = 20, **kwargs) -> None:
@@ -252,15 +330,14 @@ class PyTorchClassifier(ClassGradientsMixin, ClassifierMixin, PyTorchEstimator):
         import torch  # lgtm [py/repeated-import]
         from art.data_generators import PyTorchDataGenerator
 
+        # Put the model in the training mode
+        self._model.train()
+
         if self._optimizer is None:
             raise ValueError("An optimizer is needed to train the model, but none for provided.")
 
         # Train directly in PyTorch
-        if (
-            isinstance(generator, PyTorchDataGenerator)
-            and (self.preprocessing_defences is None or self.preprocessing_defences == [])
-            and self.preprocessing == (0, 1)
-        ):
+        if isinstance(generator, PyTorchDataGenerator) and not self.preprocessing:
             for _ in range(nb_epochs):
                 for i_batch, o_batch in generator.iterator:
                     if isinstance(i_batch, np.ndarray):
@@ -282,12 +359,21 @@ class PyTorchClassifier(ClassGradientsMixin, ClassifierMixin, PyTorchEstimator):
                     # Form the loss function
                     loss = self._loss(model_outputs[-1], o_batch)
 
-                    # Actual training
-                    loss.backward()
+                    # Do training
+                    if self._use_amp:
+                        from apex import amp
+
+                        with amp.scale_loss(loss, self._optimizer) as scaled_loss:
+                            scaled_loss.backward()
+
+                    else:
+                        loss.backward()
+
                     self._optimizer.step()
+
         else:
             # Fit a generic data generator through the API
-            super(PyTorchClassifier, self).fit_generator(generator, nb_epochs=nb_epochs)
+            super().fit_generator(generator, nb_epochs=nb_epochs)
 
     def class_gradient(self, x: np.ndarray, label: Union[int, List[int], None] = None, **kwargs) -> np.ndarray:
         """
@@ -317,21 +403,26 @@ class PyTorchClassifier(ClassGradientsMixin, ClassifierMixin, PyTorchEstimator):
             raise ValueError("Label %s is out of range." % label)
 
         # Apply preprocessing
-        x_preprocessed, _ = self._apply_preprocessing(x, y=None, fit=False)
-        x_preprocessed = torch.from_numpy(x_preprocessed).to(self._device)
-
-        # Compute gradients
-        if self._layer_idx_gradients < 0:
-            x_preprocessed.requires_grad = True
+        if self.all_framework_preprocessing:
+            x_grad = torch.from_numpy(x).to(self._device)
+            if self._layer_idx_gradients < 0:
+                x_grad.requires_grad = True
+            x_input, _ = self._apply_preprocessing(x_grad, y=None, fit=False, no_grad=False)
+        else:
+            x_preprocessed, _ = self._apply_preprocessing(x, y=None, fit=False, no_grad=True)
+            x_grad = torch.from_numpy(x_preprocessed).to(self._device)
+            if self._layer_idx_gradients < 0:
+                x_grad.requires_grad = True
+            x_input = x_grad
 
         # Run prediction
-        model_outputs = self._model(x_preprocessed)
+        model_outputs = self._model(x_input)
 
         # Set where to get gradient
         if self._layer_idx_gradients >= 0:
             input_grad = model_outputs[self._layer_idx_gradients]
         else:
-            input_grad = x_preprocessed
+            input_grad = x_grad
 
         # Set where to get gradient from
         preds = model_outputs[-1]
@@ -373,11 +464,52 @@ class PyTorchClassifier(ClassGradientsMixin, ClassifierMixin, PyTorchEstimator):
             grads = grads[None, ...]
 
         grads = np.swapaxes(np.array(grads), 0, 1)
-        grads = self._apply_preprocessing_gradient(x, grads)
+        if not self.all_framework_preprocessing:
+            grads = self._apply_preprocessing_gradient(x, grads)
 
         return grads
 
-    def loss_gradient(self, x: np.ndarray, y: np.ndarray, **kwargs) -> np.ndarray:
+    def loss(self, x: np.ndarray, y: np.ndarray, reduction: str = "none", **kwargs) -> np.ndarray:
+        """
+        Compute the loss function w.r.t. `x`.
+
+        :param x: Sample input with shape as expected by the model.
+        :param y: Target values (class labels) one-hot-encoded of shape `(nb_samples, nb_classes)` or indices
+                  of shape `(nb_samples,)`.
+        :param reduction: Specifies the reduction to apply to the output: 'none' | 'mean' | 'sum'.
+                   'none': no reduction will be applied
+                   'mean': the sum of the output will be divided by the number of elements in the output,
+                   'sum': the output will be summed.
+        :return: Array of losses of the same shape as `x`.
+        """
+        import torch  # lgtm [py/repeated-import]
+
+        # Apply preprocessing
+        x_preprocessed, y_preprocessed = self._apply_preprocessing(x, y, fit=False)
+
+        # Check label shape
+        y_preprocessed = self.reduce_labels(y_preprocessed)
+
+        # Convert the inputs to Tensors
+        inputs_t = torch.from_numpy(x_preprocessed).to(self._device)
+
+        # Convert the labels to Tensors
+        labels_t = torch.from_numpy(y_preprocessed).to(self._device)
+
+        # Compute the loss and return
+        model_outputs = self._model(inputs_t)
+        prev_reduction = self._loss.reduction
+
+        # Return individual loss values
+        self._loss.reduction = reduction
+        loss = self._loss(model_outputs[-1], labels_t)
+        self._loss.reduction = prev_reduction
+
+        return loss.detach().numpy()
+
+    def loss_gradient(
+        self, x: Union[np.ndarray, "torch.Tensor"], y: Union[np.ndarray, "torch.Tensor"], **kwargs
+    ) -> Union[np.ndarray, "torch.Tensor"]:
         """
         Compute the gradient of the loss function w.r.t. `x`.
 
@@ -389,18 +521,26 @@ class PyTorchClassifier(ClassGradientsMixin, ClassifierMixin, PyTorchEstimator):
         import torch  # lgtm [py/repeated-import]
 
         # Apply preprocessing
-        x_preprocessed, y_preprocessed = self._apply_preprocessing(x, y, fit=False)
+        if self.all_framework_preprocessing:
+            x_grad = torch.tensor(x).to(self._device)
+            y_grad = torch.tensor(y).to(self._device)
+            x_grad.requires_grad = True
+            inputs_t, y_preprocessed = self._apply_preprocessing(x_grad, y=y_grad, fit=False, no_grad=False)
+        elif isinstance(x, np.ndarray):
+            x_preprocessed, y_preprocessed = self._apply_preprocessing(x, y=y, fit=False, no_grad=True)
+            x_grad = torch.from_numpy(x_preprocessed).to(self._device)
+            x_grad.requires_grad = True
+            inputs_t = x_grad
+        else:
+            raise NotImplementedError("Combination of inputs and preprocessing not supported.")
 
         # Check label shape
-        if self._reduce_labels:
-            y_preprocessed = np.argmax(y_preprocessed, axis=1)
+        y_preprocessed = self.reduce_labels(y_preprocessed)
 
-        # Convert the inputs to Tensors
-        inputs_t = torch.from_numpy(x_preprocessed).to(self._device)
-        inputs_t.requires_grad = True
-
-        # Convert the labels to Tensors
-        labels_t = torch.from_numpy(y_preprocessed).to(self._device)
+        if isinstance(y_preprocessed, np.ndarray):
+            labels_t = torch.from_numpy(y_preprocessed).to(self._device)
+        else:
+            labels_t = y_preprocessed
 
         # Compute the gradient and return
         model_outputs = self._model(inputs_t)
@@ -410,46 +550,27 @@ class PyTorchClassifier(ClassGradientsMixin, ClassifierMixin, PyTorchEstimator):
         self._model.zero_grad()
 
         # Compute gradients
-        loss.backward()
-        grads = inputs_t.grad.cpu().numpy().copy()  # type: ignore
-        grads = self._apply_preprocessing_gradient(x, grads)
+        if self._use_amp:
+            from apex import amp
+
+            with amp.scale_loss(loss, self._optimizer) as scaled_loss:
+                scaled_loss.backward()
+
+        else:
+            loss.backward()
+
+        if isinstance(x, torch.Tensor):
+            grads = x_grad.grad
+        else:
+            grads = x_grad.grad.cpu().numpy().copy()  # type: ignore
+
+        if not self.all_framework_preprocessing:
+            grads = self._apply_preprocessing_gradient(x, grads)
+
         assert grads.shape == x.shape
 
         return grads
 
-    def loss_gradient_framework(self, x: "torch.Tensor", y: "torch.Tensor", **kwargs) -> "torch.Tensor":
-        """
-        Compute the gradient of the loss function w.r.t. `x`.
-
-        :param x: Input with shape as expected by the model.
-        :param y: Target values (class labels) one-hot-encoded of shape (nb_samples, nb_classes) or indices of shape
-                  (nb_samples,).
-        :return: Gradients of the same shape as `x`.
-        """
-        import torch  # lgtm [py/repeated-import]
-        from torch.autograd import Variable
-
-        # Check label shape
-        if self._reduce_labels:
-            y = torch.argmax(y, dim=1)
-
-        # Convert the inputs to Variable
-        x = Variable(x, requires_grad=True)
-
-        # Compute the gradient and return
-        model_outputs = self._model(x)
-        loss = self._loss(model_outputs[-1], y)
-
-        # Clean gradients
-        self._model.zero_grad()
-
-        # Compute gradients
-        loss.backward()
-        grads = x.grad
-        assert grads.shape == x.shape  # type: ignore
-
-        return grads  # type: ignore
-
     def get_activations(
         self, x: np.ndarray, layer: Union[int, str], batch_size: int = 128, framework: bool = False
     ) -> np.ndarray:
@@ -500,6 +621,7 @@ class PyTorchClassifier(ClassGradientsMixin, ClassifierMixin, PyTorchEstimator):
             results.append(layer_output.detach().cpu().numpy())
 
         results = np.concatenate(results)
+
         return results
 
     def set_learning_phase(self, train: bool) -> None:
@@ -523,7 +645,7 @@ class PyTorchClassifier(ClassGradientsMixin, ClassifierMixin, PyTorchEstimator):
         import torch  # lgtm [py/repeated-import]
 
         if path is None:
-            full_path = os.path.join(ART_DATA_PATH, filename)
+            full_path = os.path.join(config.ART_DATA_PATH, filename)
         else:
             full_path = os.path.join(path, filename)
         folder = os.path.split(full_path)[0]
@@ -539,7 +661,7 @@ class PyTorchClassifier(ClassGradientsMixin, ClassifierMixin, PyTorchEstimator):
 
     def __getstate__(self) -> Dict[str, Any]:
         """
-        Use to ensure `PytorchClassifier` can be pickled.
+        Use to ensure `PyTorchClassifier` can be pickled.
 
         :return: State dictionary with instance parameters.
         """
@@ -561,7 +683,7 @@ class PyTorchClassifier(ClassGradientsMixin, ClassifierMixin, PyTorchEstimator):
 
     def __setstate__(self, state: Dict[str, Any]) -> None:
         """
-        Use to ensure `PytorchClassifier` can be unpickled.
+        Use to ensure `PyTorchClassifier` can be unpickled.
 
         :param state: State dictionary with instance parameters to restore.
         """
@@ -569,7 +691,7 @@ class PyTorchClassifier(ClassGradientsMixin, ClassifierMixin, PyTorchEstimator):
 
         # Recover model
         self.__dict__.update(state)
-        full_path = os.path.join(ART_DATA_PATH, state["model_name"])
+        full_path = os.path.join(config.ART_DATA_PATH, state["model_name"])
         model = state["inner_model"]
         model.load_state_dict(torch.load(str(full_path) + ".model"))
         model.eval()
@@ -607,7 +729,7 @@ class PyTorchClassifier(ClassGradientsMixin, ClassifierMixin, PyTorchEstimator):
 
         return repr_
 
-    def _make_model_wrapper(self, model: "torch.nn.Module"):
+    def _make_model_wrapper(self, model: "torch.nn.Module") -> "torch.nn.Module":
         # Try to import PyTorch and create an internal class that acts like a model wrapper extending torch.nn.Module
         try:
             import torch.nn as nn
@@ -620,13 +742,15 @@ class PyTorchClassifier(ClassGradientsMixin, ClassifierMixin, PyTorchEstimator):
                     This is a wrapper for the input model.
                     """
 
-                    def __init__(self, model: "torch.nn.Module"):
+                    import torch  # lgtm [py/repeated-import]
+
+                    def __init__(self, model: torch.nn.Module):
                         """
                         Initialization by storing the input model.
 
                         :param model: PyTorch model. The forward function of the model must return the logit output.
                         """
-                        super(ModelWrapper, self).__init__()
+                        super().__init__()
                         self._model = model
 
                     # pylint: disable=W0221
@@ -700,4 +824,4 @@ class PyTorchClassifier(ClassGradientsMixin, ClassifierMixin, PyTorchEstimator):
             return self._model_wrapper(model)
 
         except ImportError:
-            raise ImportError("Could not find PyTorch (`torch`) installation.")
+            raise ImportError("Could not find PyTorch (`torch`) installation.") from ImportError
