@@ -24,14 +24,16 @@ from typing import Any, Dict, List, Optional, Tuple, Union, TYPE_CHECKING
 import numpy as np
 from tqdm import trange
 
-from art.config import ART_NUMPY_DTYPE, CLIP_VALUES_TYPE, PREPROCESSING_TYPE
-from art.defences.postprocessor.postprocessor import Postprocessor
-from art.defences.preprocessor.preprocessor import Preprocessor
+from art.config import ART_NUMPY_DTYPE
 from art.utils import Deprecated, deprecated, deprecated_keyword_arg
 
 if TYPE_CHECKING:
+    # pylint: disable=R0401
+    from art.utils import CLIP_VALUES_TYPE, PREPROCESSING_TYPE
     from art.data_generators import DataGenerator
     from art.metrics.verification_decisions_trees import Tree
+    from art.defences.postprocessor.postprocessor import Postprocessor
+    from art.defences.preprocessor.preprocessor import Preprocessor
 
 
 class BaseEstimator(ABC):
@@ -50,11 +52,11 @@ class BaseEstimator(ABC):
 
     def __init__(
         self,
-        model=None,
-        clip_values: Optional[CLIP_VALUES_TYPE] = None,
+        model,
+        clip_values: Optional["CLIP_VALUES_TYPE"],
         preprocessing_defences: Union["Preprocessor", List["Preprocessor"], None] = None,
         postprocessing_defences: Union["Postprocessor", List["Postprocessor"], None] = None,
-        preprocessing: PREPROCESSING_TYPE = (0, 1),
+        preprocessing: "PREPROCESSING_TYPE" = (0, 1),
     ):
         """
         Initialize a `BaseEstimator` object.
@@ -66,26 +68,44 @@ class BaseEstimator(ABC):
                the shape of clip values needs to match the total number of features.
         :param preprocessing_defences: Preprocessing defence(s) to be applied by the estimator.
         :param postprocessing_defences: Postprocessing defence(s) to be applied by the estimator.
-        :param preprocessing: Tuple of the form `(subtractor, divider)` of floats or `np.ndarray` of values to be
+        :param preprocessing: Tuple of the form `(subtrahend, divisor)` of floats or `np.ndarray` of values to be
                used for data preprocessing. The first value will be subtracted from the input and the results will be
                divided by the second value.
         """
+        from art.defences.postprocessor.postprocessor import Postprocessor
+        from art.defences.preprocessor.preprocessor import Preprocessor
+
         self._model = model
         self._clip_values = clip_values
 
-        self.preprocessing_defences: Optional[List[Preprocessor]]
-        if isinstance(preprocessing_defences, Preprocessor):
-            self.preprocessing_defences = [preprocessing_defences]
-        else:
-            self.preprocessing_defences = preprocessing_defences
+        self.preprocessing: List["Preprocessor"] = []
 
-        self.postprocessing_defences: Optional[List[Postprocessor]]
+        if preprocessing_defences is None:
+            pass
+        elif isinstance(preprocessing_defences, Preprocessor):
+            self.preprocessing.append(preprocessing_defences)
+        else:
+            self.preprocessing += preprocessing_defences
+
+        self.preprocessing_defences = preprocessing_defences
+
+        if preprocessing is None:
+            pass
+        elif isinstance(preprocessing, tuple):
+            from art.preprocessing.standardisation_mean_std.standardisation_mean_std import StandardisationMeanStd
+
+            self.preprocessing.append(StandardisationMeanStd(mean=preprocessing[0], std=preprocessing[1]))
+        elif isinstance(preprocessing, Preprocessor):
+            self.preprocessing.append(preprocessing)
+        else:
+            self.preprocessing += preprocessing
+
+        self.postprocessing_defences: Optional[List["Postprocessor"]]
         if isinstance(postprocessing_defences, Postprocessor):
             self.postprocessing_defences = [postprocessing_defences]
         else:
             self.postprocessing_defences = postprocessing_defences
 
-        self.preprocessing = preprocessing
         self._check_params()
 
     def set_params(self, **kwargs) -> None:
@@ -116,6 +136,9 @@ class BaseEstimator(ABC):
         return params
 
     def _check_params(self) -> None:
+        from art.defences.postprocessor.postprocessor import Postprocessor
+        from art.defences.preprocessor.preprocessor import Preprocessor
+
         if self._clip_values is not None:
             if len(self._clip_values) != 2:
                 raise ValueError(
@@ -129,20 +152,19 @@ class BaseEstimator(ABC):
             else:
                 self._clip_values = np.array(self._clip_values, dtype=ART_NUMPY_DTYPE)
 
-        if isinstance(self.preprocessing_defences, list):
-            for preproc_defence in self.preprocessing_defences:
-                if not isinstance(preproc_defence, Preprocessor):
+        if isinstance(self.preprocessing, list):
+            for preprocess in self.preprocessing:
+                if not isinstance(preprocess, Preprocessor):
                     raise ValueError(
                         "All preprocessing defences have to be instance of "
                         "art.defences.preprocessor.preprocessor.Preprocessor."
                     )
-        elif self.preprocessing_defences is None:
-            pass
         else:
             raise ValueError(
                 "All preprocessing defences have to be instance of "
                 "art.defences.preprocessor.preprocessor.Preprocessor."
             )
+
         if isinstance(self.postprocessing_defences, list):
             for postproc_defence in self.postprocessing_defences:
                 if not isinstance(postproc_defence, Postprocessor):
@@ -158,19 +180,14 @@ class BaseEstimator(ABC):
                 "art.defences.postprocessor.postprocessor.Postprocessor."
             )
 
-        if self.preprocessing is not None and len(self.preprocessing) != 2:
-            raise ValueError(
-                "`preprocessing` should be a tuple of 2 floats with the values to subtract and divide the model inputs."
-            )
-
     @abstractmethod
-    def predict(self, x, *args, **kwargs):  # lgtm [py/inheritance/incorrect-overridden-signature]
+    def predict(self, x, **kwargs) -> Any:  # lgtm [py/inheritance/incorrect-overridden-signature]
         """
         Perform prediction of the estimator for input `x`.
 
         :param x: Samples.
         :type x: Format as expected by the `model`
-        :return: Array of predictions by the model.
+        :return: Predictions by the model.
         :rtype: Format as produced by the `model`
         """
         raise NotImplementedError
@@ -197,16 +214,17 @@ class BaseEstimator(ABC):
         return self._model
 
     @property
+    @abstractmethod
     def input_shape(self) -> Tuple[int, ...]:
         """
         Return the shape of one input sample.
 
         :return: Shape of one input sample.
         """
-        return self._input_shape  # type: ignore
+        raise NotImplementedError
 
     @property
-    def clip_values(self) -> Optional[CLIP_VALUES_TYPE]:
+    def clip_values(self) -> Optional["CLIP_VALUES_TYPE"]:
         """
         Return the clip values of the input samples.
 
@@ -227,68 +245,17 @@ class BaseEstimator(ABC):
         :return: Tuple of `x` and `y` after applying the defences and standardisation.
         :rtype: Format as expected by the `model`
         """
-        # y = check_and_transform_label_format(y, self.nb_classes)
-        x_preprocessed, y_preprocessed = self._apply_preprocessing_defences(x, y, fit=fit)
-        x_preprocessed = self._apply_preprocessing_standardisation(x_preprocessed)
-        return x_preprocessed, y_preprocessed
-
-    def _apply_preprocessing_defences(self, x, y, fit: bool = False) -> Tuple[Any, Any]:
-        """
-        Apply all preprocessing defences of the estimator on the raw inputs `x` and `y`. This function is should
-        only be called from function `_apply_preprocessing`.
-
-        :param x: Samples.
-        :type x: Format as expected by the `model`
-        :param y: Target values.
-        :type y: Format as expected by the `model`
-        :param fit: `True` if the function is call before fit/training and `False` if the function is called before a
-                    predict operation.
-        :return: Tuple of `x` and `y` after applying the defences and standardisation.
-        :rtype: Format as expected by the `model`
-        """
-        if self.preprocessing_defences is not None:
-            for defence in self.preprocessing_defences:
+        if self.preprocessing:
+            for preprocess in self.preprocessing:
                 if fit:
-                    if defence.apply_fit:
-                        x, y = defence(x, y)
+                    if preprocess.apply_fit:
+                        x, y = preprocess(x, y)
                 else:
-                    if defence.apply_predict:
-                        x, y = defence(x, y)
+                    if preprocess.apply_predict:
+                        x, y = preprocess(x, y)
 
         return x, y
 
-    def _apply_preprocessing_standardisation(self, x):
-        """
-        Apply standardisation to input data `x`.
-
-        :param x: Samples.
-        :type x: Format as expected by the `model`
-        :return: Standardized `x`.
-        :rtype: Format as expected by the `model`
-        :raises `TypeError`: If the input data type is unsigned.
-        """
-        if x.dtype in [np.uint8, np.uint16, np.uint32, np.uint64]:
-            raise TypeError(
-                "The data type of input data `x` is {} and cannot represent negative values. Consider "
-                "changing the data type of the input data `x` to a type that supports negative values e.g. "
-                "np.float32.".format(x.dtype)
-            )
-
-        if self.preprocessing is not None:
-            sub, div = self.preprocessing
-
-            if isinstance(x, np.ndarray):
-                sub = np.asarray(sub, dtype=x.dtype)
-                div = np.asarray(div, dtype=x.dtype)
-
-            res = x - sub
-            res = res / div
-
-        else:
-            res = x
-
-        return res
-
     def _apply_postprocessing(self, preds, fit: bool) -> np.ndarray:
         """
         Apply all postprocessing defences on model predictions.
@@ -342,7 +309,7 @@ class LossGradientsMixin(ABC):
         """
         raise NotImplementedError
 
-    def _apply_preprocessing_gradient(self, x, gradients):
+    def _apply_preprocessing_gradient(self, x, gradients, fit=False):
         """
         Apply the backward pass to the gradients through all normalization and preprocessing defences that have been
         applied to `x` and `y` in the forward pass. This function has to be applied to all gradients w.r.t. `x`
@@ -355,55 +322,17 @@ class LossGradientsMixin(ABC):
         :return: Gradients after backward pass through normalization and preprocessing defences.
         :rtype: Format as expected by the `model`
         """
-        gradients = self._apply_preprocessing_normalization_gradient(gradients)
-        gradients = self._apply_preprocessing_defences_gradient(x, gradients)
-        return gradients
-
-    def _apply_preprocessing_defences_gradient(self, x, gradients, fit=False):
-        """
-        Apply the backward pass to the gradients through all preprocessing defences that have been applied to `x`
-        and `y` in the forward pass. This function is should only be called from function
-        `_apply_preprocessing_gradient`.
-
-        :param x: Samples.
-        :type x: Format as expected by the `model`
-        :param gradients: Gradients before backward pass through preprocessing defences.
-        :type gradients: Format as expected by the `model`
-        :param fit: `True` if the gradients are computed during training.
-        :return: Gradients after backward pass through preprocessing defences.
-        :rtype: Format as expected by the `model`
-        """
-        if hasattr(self, "preprocessing_defences") and self.preprocessing_defences is not None:
-            for defence in self.preprocessing_defences[::-1]:
+        if self.preprocessing:
+            for preprocess in self.preprocessing[::-1]:
                 if fit:
-                    if defence.apply_fit:
-                        gradients = defence.estimate_gradient(x, gradients)
+                    if preprocess.apply_fit:
+                        gradients = preprocess.estimate_gradient(x, gradients)
                 else:
-                    if defence.apply_predict:
-                        gradients = defence.estimate_gradient(x, gradients)
+                    if preprocess.apply_predict:
+                        gradients = preprocess.estimate_gradient(x, gradients)
 
         return gradients
 
-    def _apply_preprocessing_normalization_gradient(self, gradients):
-        """
-        Apply the backward pass through standardisation of `x` to `gradients`.
-
-        Apply the backward pass to the gradients through normalization that has been applied to `x` in the forward
-        pass. This function is should only be called from function `_apply_preprocessing_gradient`.
-
-        :param gradients: Gradients before backward pass through normalization.
-        :type gradients: Format as expected by the `model`
-        :return: Gradients after backward pass through normalization.
-        """
-        if hasattr(self, "preprocessing") and self.preprocessing is not None:
-            _, div = self.preprocessing
-            div = np.asarray(div, dtype=gradients.dtype)
-            res = gradients / div
-        else:
-            res = gradients
-
-        return res
-
 
 class NeuralNetworkMixin(ABC):
     """
@@ -411,8 +340,8 @@ class NeuralNetworkMixin(ABC):
     has to be mixed in with class `BaseEstimator`.
     """
 
-    @deprecated_keyword_arg("channel_index", end_version="1.5.0", replaced_by="channels_first")
-    def __init__(self, channel_index=Deprecated, channels_first: Optional[bool] = None, **kwargs) -> None:
+    @deprecated_keyword_arg("channel_index", end_version="1.6.0", replaced_by="channels_first")
+    def __init__(self, channels_first: Optional[bool], channel_index=Deprecated, **kwargs) -> None:
         """
         Initialize a neural network attributes.
 
@@ -420,7 +349,7 @@ class NeuralNetworkMixin(ABC):
         :type channel_index: `int`
         :param channels_first: Set channels first or last.
         """
-        # Remove in 1.5.0
+        # Remove in 1.6.0
         if channel_index == 3:
             channels_first = False
         elif channel_index == 1:
@@ -512,8 +441,22 @@ class NeuralNetworkMixin(ABC):
         """
         raise NotImplementedError
 
+    @abstractmethod
+    def loss(self, x: np.ndarray, y: np.ndarray, **kwargs) -> np.ndarray:
+        """
+        Compute the loss of the neural network for samples `x`.
+
+        :param x: Samples of shape (nb_samples, nb_features) or (nb_samples, nb_pixels_1, nb_pixels_2,
+                  nb_channels) or (nb_samples, nb_channels, nb_pixels_1, nb_pixels_2).
+        :param y: Target values (class labels) one-hot-encoded of shape `(nb_samples, nb_classes)` or indices
+                  of shape `(nb_samples,)`.
+        :return: Loss values.
+        :rtype: Format as expected by the `model`
+        """
+        raise NotImplementedError
+
     @property  # type: ignore
-    @deprecated(end_version="1.5.0", replaced_by="channels_first")
+    @deprecated(end_version="1.6.0", replaced_by="channels_first")
     def channel_index(self) -> Optional[int]:
         """
         :return: Index of the axis containing the color channels in the samples `x`.
@@ -540,7 +483,7 @@ class NeuralNetworkMixin(ABC):
         return self._learning_phase  # type: ignore
 
     @property
-    def layer_names(self) -> List[str]:
+    def layer_names(self) -> Optional[List[str]]:
         """
         Return the names of the hidden layers in the model, if applicable.
 
