@@ -6,4 +6,6 @@ from art.metrics.metrics import loss_sensitivity
 from art.metrics.metrics import clever
 from art.metrics.metrics import clever_u
 from art.metrics.metrics import clever_t
+from art.metrics.metrics import wasserstein_distance
 from art.metrics.verification_decisions_trees import RobustnessVerificationTreeModelsCliqueMethod
+from art.metrics.gradient_check import loss_gradient_check
