@@ -24,6 +24,7 @@ import keras.backend as k
 import numpy as np
 
 from art.attacks.evasion.elastic_net import ElasticNet
+from art.estimators.estimator import BaseEstimator
 from art.estimators.classification.classifier import ClassGradientsMixin
 from art.estimators.classification.keras import KerasClassifier
 from art.utils import random_targets, to_categorical
@@ -63,7 +64,7 @@ class TestElasticNet(TestBase):
         master_seed(seed=1234)
         super().setUp()
 
-    def test_tensorflow_failure_attack(self):
+    def test_2_tensorflow_failure_attack(self):
         """
         Test the corner case when attack fails.
         :return:
@@ -85,7 +86,7 @@ class TestElasticNet(TestBase):
         if sess is not None:
             sess.close()
 
-    def test_tensorflow_mnist(self):
+    def test_4_tensorflow_mnist(self):
         """
         First test with the TensorFlowClassifier.
         :return:
@@ -266,7 +267,7 @@ class TestElasticNet(TestBase):
         if sess is not None:
             sess.close()
 
-    def test_keras_mnist(self):
+    def test_9a_keras_mnist(self):
         """
         Second test with the KerasClassifier.
         :return:
@@ -340,7 +341,7 @@ class TestElasticNet(TestBase):
 
         k.clear_session()
 
-    def test_pytorch_mnist(self):
+    def test_6_pytorch_mnist(self):
         """
         Third test with the PyTorchClassifier.
         :return:
@@ -408,10 +409,10 @@ class TestElasticNet(TestBase):
         # Check that x_test has not been modified by attack and classifier
         self.assertAlmostEqual(float(np.max(np.abs(x_test_original - x_test))), 0.0, delta=0.00001)
 
-    def test_classifier_type_check_fail(self):
-        backend_test_classifier_type_check_fail(ElasticNet, [ClassGradientsMixin])
+    def test_1_classifier_type_check_fail(self):
+        backend_test_classifier_type_check_fail(ElasticNet, [BaseEstimator, ClassGradientsMixin])
 
-    def test_keras_iris_clipped(self):
+    def test_8_keras_iris_clipped(self):
         classifier = get_tabular_classifier_kr()
         attack = ElasticNet(classifier, targeted=False, max_iter=10)
         x_test_adv = attack.generate(self.x_test_iris)
@@ -475,7 +476,7 @@ class TestElasticNet(TestBase):
         accuracy = 1.0 - np.sum(predictions_adv == np.argmax(self.y_test_iris, axis=1)) / self.y_test_iris.shape[0]
         logger.info("EAD success rate on Iris: %.2f%%", (accuracy * 100))
 
-    def test_keras_iris_unbounded(self):
+    def test_9_keras_iris_unbounded(self):
         classifier = get_tabular_classifier_kr()
 
         # Recreate a classifier without clip values
@@ -540,7 +541,7 @@ class TestElasticNet(TestBase):
         accuracy = 1.0 - np.sum(predictions_adv == np.argmax(self.y_test_iris, axis=1)) / self.y_test_iris.shape[0]
         logger.info("EAD success rate on Iris: %.2f%%", (accuracy * 100))
 
-    def test_tensorflow_iris(self):
+    def test_3_tensorflow_iris(self):
         classifier, _ = get_tabular_classifier_tf()
 
         # Test untargeted attack
@@ -673,7 +674,7 @@ class TestElasticNet(TestBase):
         accuracy = np.sum(predictions_adv == np.argmax(targets, axis=1)) / self.y_test_iris.shape[0]
         logger.info("Targeted EAD success rate on Iris: %.2f%%", (accuracy * 100))
 
-    def test_pytorch_iris(self):
+    def test_5_pytorch_iris(self):
         classifier = get_tabular_classifier_pt()
         attack = ElasticNet(classifier, targeted=False, max_iter=10)
         x_test_adv = attack.generate(self.x_test_iris.astype(np.float32))
@@ -739,7 +740,7 @@ class TestElasticNet(TestBase):
         accuracy = 1.0 - np.sum(predictions_adv == np.argmax(self.y_test_iris, axis=1)) / self.y_test_iris.shape[0]
         logger.info("EAD success rate on Iris: %.2f%%", (accuracy * 100))
 
-    def test_scikitlearn(self):
+    def test_7_scikitlearn(self):
         from sklearn.linear_model import LogisticRegression
         from sklearn.svm import SVC, LinearSVC
 
