@@ -52,7 +52,7 @@ class TestIterativeAttack(TestBase):
         cls.x_test_mnist = cls.x_test_mnist[0 : cls.n_test]
         cls.y_test_mnist = cls.y_test_mnist[0 : cls.n_test]
 
-    def test_keras_mnist(self):
+    def test_9b_keras_mnist(self):
         classifier = get_image_classifier_kr()
 
         scores = classifier._model.evaluate(self.x_train_mnist, self.y_train_mnist)
@@ -64,7 +64,7 @@ class TestIterativeAttack(TestBase):
             classifier, self.x_train_mnist, self.y_train_mnist, self.x_test_mnist, self.y_test_mnist
         )
 
-    def test_tensorflow_mnist(self):
+    def test_3_tensorflow_mnist(self):
         classifier, sess = get_image_classifier_tf()
 
         scores = get_labels_np_array(classifier.predict(self.x_train_mnist))
@@ -79,7 +79,7 @@ class TestIterativeAttack(TestBase):
             classifier, self.x_train_mnist, self.y_train_mnist, self.x_test_mnist, self.y_test_mnist
         )
 
-    def test_pytorch_mnist(self):
+    def test_6_pytorch_mnist(self):
         classifier = get_image_classifier_pt()
         x_train = np.swapaxes(self.x_train_mnist, 1, 3).astype(np.float32)
         x_test = np.swapaxes(self.x_test_mnist, 1, 3).astype(np.float32)
@@ -98,7 +98,7 @@ class TestIterativeAttack(TestBase):
         x_test_original = x_test.copy()
 
         # Test BIM with np.inf norm
-        attack = BasicIterativeMethod(classifier, eps=1, eps_step=0.1, batch_size=128)
+        attack = BasicIterativeMethod(classifier, eps=1.0, eps_step=0.1, batch_size=128)
         x_train_adv = attack.generate(x_train)
         x_test_adv = attack.generate(x_test)
 
@@ -120,6 +120,58 @@ class TestIterativeAttack(TestBase):
         # Check that x_test has not been modified by attack and classifier
         self.assertAlmostEqual(float(np.max(np.abs(x_test_original - x_test))), 0.0, delta=0.00001)
 
+        # Test eps of array type 1
+        eps = np.ones(shape=x_test.shape) * 1.0
+        eps_step = np.ones_like(eps) * 0.1
+
+        attack_params = {"eps_step": eps_step, "eps": eps}
+        attack.set_params(**attack_params)
+
+        x_test_adv = attack.generate(x_test)
+        self.assertFalse((x_test == x_test_adv).all())
+
+        test_y_pred = get_labels_np_array(classifier.predict(x_test_adv))
+        self.assertFalse((y_test == test_y_pred).all())
+
+        # Test eps of array type 2
+        eps = np.ones(shape=x_test.shape[1:]) * 1.0
+        eps_step = np.ones_like(eps) * 0.1
+
+        attack_params = {"eps_step": eps_step, "eps": eps}
+        attack.set_params(**attack_params)
+
+        x_test_adv = attack.generate(x_test)
+        self.assertFalse((x_test == x_test_adv).all())
+
+        test_y_pred = get_labels_np_array(classifier.predict(x_test_adv))
+        self.assertFalse((y_test == test_y_pred).all())
+
+        # Test eps of array type 3
+        eps = np.ones(shape=x_test.shape[2:]) * 1.0
+        eps_step = np.ones_like(eps) * 0.1
+
+        attack_params = {"eps_step": eps_step, "eps": eps}
+        attack.set_params(**attack_params)
+
+        x_test_adv = attack.generate(x_test)
+        self.assertFalse((x_test == x_test_adv).all())
+
+        test_y_pred = get_labels_np_array(classifier.predict(x_test_adv))
+        self.assertFalse((y_test == test_y_pred).all())
+
+        # Test eps of array type 4
+        eps = np.ones(shape=x_test.shape[3:]) * 1.0
+        eps_step = np.ones_like(eps) * 0.1
+
+        attack_params = {"eps_step": eps_step, "eps": eps}
+        attack.set_params(**attack_params)
+
+        x_test_adv = attack.generate(x_test)
+        self.assertFalse((x_test == x_test_adv).all())
+
+        test_y_pred = get_labels_np_array(classifier.predict(x_test_adv))
+        self.assertFalse((y_test == test_y_pred).all())
+
     def _test_mnist_targeted(self, classifier, x_test):
         x_test_original = x_test.copy()
 
@@ -143,27 +195,27 @@ class TestIterativeAttack(TestBase):
         # Check that x_test has not been modified by attack and classifier
         self.assertAlmostEqual(float(np.max(np.abs(x_test_original - x_test))), 0.0, delta=0.00001)
 
-    def test_keras_mnist_targeted(self):
+    def test_9c_keras_mnist_targeted(self):
         classifier = get_image_classifier_kr()
         self._test_mnist_targeted(classifier, self.x_test_mnist)
 
-    def test_tensorflow_mnist_targeted(self):
+    def test_4_tensorflow_mnist_targeted(self):
         classifier, sess = get_image_classifier_tf()
         self._test_mnist_targeted(classifier, self.x_test_mnist)
 
-    def test_pytorch_mnist_targeted(self):
+    def test_7_pytorch_mnist_targeted(self):
         classifier = get_image_classifier_pt()
         x_test = np.swapaxes(self.x_test_mnist, 1, 3).astype(np.float32)
         self._test_mnist_targeted(classifier, x_test)
 
-    def test_classifier_type_check_fail(self):
+    def test_1_classifier_type_check_fail(self):
         backend_test_classifier_type_check_fail(BasicIterativeMethod, [BaseEstimator, LossGradientsMixin])
 
-    def test_keras_iris_clipped(self):
+    def test_9_keras_iris_clipped(self):
         classifier = get_tabular_classifier_kr()
 
         # Test untargeted attack
-        attack = BasicIterativeMethod(classifier, eps=1, eps_step=0.1, batch_size=128)
+        attack = BasicIterativeMethod(classifier, eps=1.0, eps_step=0.1, batch_size=128)
         x_test_adv = attack.generate(self.x_test_iris)
         self.assertFalse((self.x_test_iris == x_test_adv).all())
         self.assertTrue((x_test_adv <= 1).all())
@@ -176,7 +228,7 @@ class TestIterativeAttack(TestBase):
 
         # Test targeted attack
         targets = random_targets(self.y_test_iris, nb_classes=3)
-        attack = BasicIterativeMethod(classifier, targeted=True, eps=1, eps_step=0.1)
+        attack = BasicIterativeMethod(classifier, targeted=True, eps=1.0, eps_step=0.1)
         x_test_adv = attack.generate(self.x_test_iris, **{"y": targets})
         self.assertFalse((self.x_test_iris == x_test_adv).all())
         self.assertTrue((x_test_adv <= 1).all())
@@ -187,12 +239,12 @@ class TestIterativeAttack(TestBase):
         acc = np.sum(preds_adv == np.argmax(targets, axis=1)) / self.y_test_iris.shape[0]
         logger.info("Success rate of targeted BIM on Iris: %.2f%%", (acc * 100))
 
-    def test_keras_iris_unbounded(self):
+    def test_9a_keras_iris_unbounded(self):
         classifier = get_tabular_classifier_kr()
 
         # Recreate a classifier without clip values
         classifier = KerasClassifier(model=classifier._model, use_logits=False, channels_first=True)
-        attack = BasicIterativeMethod(classifier, eps=1, eps_step=0.2, batch_size=128)
+        attack = BasicIterativeMethod(classifier, eps=1.0, eps_step=0.2, batch_size=128)
         x_test_adv = attack.generate(self.x_test_iris)
         self.assertFalse((self.x_test_iris == x_test_adv).all())
         self.assertTrue((x_test_adv > 1).any())
@@ -203,11 +255,11 @@ class TestIterativeAttack(TestBase):
         acc = np.sum(preds_adv == np.argmax(self.y_test_iris, axis=1)) / self.y_test_iris.shape[0]
         logger.info("Accuracy on Iris with BIM adversarial examples: %.2f%%", (acc * 100))
 
-    def test_tensorflow_iris(self):
+    def test_2_tensorflow_iris(self):
         classifier, _ = get_tabular_classifier_tf()
 
         # Test untargeted attack
-        attack = BasicIterativeMethod(classifier, eps=1, eps_step=0.1, max_iter=5)
+        attack = BasicIterativeMethod(classifier, eps=1.0, eps_step=0.1, max_iter=5)
         x_test_adv = attack.generate(self.x_test_iris)
         self.assertFalse((self.x_test_iris == x_test_adv).all())
         self.assertTrue((x_test_adv <= 1).all())
@@ -220,7 +272,7 @@ class TestIterativeAttack(TestBase):
 
         # Test targeted attack
         targets = random_targets(self.y_test_iris, nb_classes=3)
-        attack = BasicIterativeMethod(classifier, targeted=True, eps=1, eps_step=0.1, max_iter=5)
+        attack = BasicIterativeMethod(classifier, targeted=True, eps=1.0, eps_step=0.1, max_iter=5)
         x_test_adv = attack.generate(self.x_test_iris, **{"y": targets})
         self.assertFalse((self.x_test_iris == x_test_adv).all())
         self.assertTrue((x_test_adv <= 1).all())
@@ -231,11 +283,11 @@ class TestIterativeAttack(TestBase):
         acc = np.sum(preds_adv == np.argmax(targets, axis=1)) / self.y_test_iris.shape[0]
         logger.info("Success rate of targeted BIM on Iris: %.2f%%", (acc * 100))
 
-    def test_pytorch_iris(self):
+    def test_5_pytorch_iris(self):
         classifier = get_tabular_classifier_pt()
 
         # Test untargeted attack
-        attack = BasicIterativeMethod(classifier, eps=1, eps_step=0.1)
+        attack = BasicIterativeMethod(classifier, eps=1.0, eps_step=0.1)
         x_test_adv = attack.generate(self.x_test_iris)
         self.assertFalse((self.x_test_iris == x_test_adv).all())
         self.assertTrue((x_test_adv <= 1).all())
@@ -248,7 +300,7 @@ class TestIterativeAttack(TestBase):
 
         # Test targeted attack
         targets = random_targets(self.y_test_iris, nb_classes=3)
-        attack = BasicIterativeMethod(classifier, targeted=True, eps=1, eps_step=0.1, batch_size=128)
+        attack = BasicIterativeMethod(classifier, targeted=True, eps=1.0, eps_step=0.1, batch_size=128)
         x_test_adv = attack.generate(self.x_test_iris, **{"y": targets})
         self.assertFalse((self.x_test_iris == x_test_adv).all())
         self.assertTrue((x_test_adv <= 1).all())
@@ -259,7 +311,7 @@ class TestIterativeAttack(TestBase):
         acc = np.sum(preds_adv == np.argmax(targets, axis=1)) / self.y_test_iris.shape[0]
         logger.info("Success rate of targeted BIM on Iris: %.2f%%", (acc * 100))
 
-    def test_scikitlearn(self):
+    def test_8_scikitlearn(self):
         from sklearn.linear_model import LogisticRegression
         from sklearn.svm import SVC, LinearSVC
 
@@ -278,7 +330,7 @@ class TestIterativeAttack(TestBase):
             classifier.fit(x=self.x_test_iris, y=self.y_test_iris)
 
             # Test untargeted attack
-            attack = BasicIterativeMethod(classifier, eps=1, eps_step=0.1, max_iter=5)
+            attack = BasicIterativeMethod(classifier, eps=1.0, eps_step=0.1, max_iter=5)
             x_test_adv = attack.generate(self.x_test_iris)
             self.assertFalse((self.x_test_iris == x_test_adv).all())
             self.assertTrue((x_test_adv <= 1).all())
@@ -294,7 +346,7 @@ class TestIterativeAttack(TestBase):
 
             # Test targeted attack
             targets = random_targets(self.y_test_iris, nb_classes=3)
-            attack = BasicIterativeMethod(classifier, targeted=True, eps=1, eps_step=0.1, batch_size=128, max_iter=5)
+            attack = BasicIterativeMethod(classifier, targeted=True, eps=1.0, eps_step=0.1, batch_size=128, max_iter=5)
             x_test_adv = attack.generate(self.x_test_iris, **{"y": targets})
             self.assertFalse((self.x_test_iris == x_test_adv).all())
             self.assertTrue((x_test_adv <= 1).all())
