@@ -58,7 +58,7 @@ class TestThresholdAttack(TestBase):
         cls.x_test_mnist = cls.x_test_mnist[0 : cls.n_test]
         cls.y_test_mnist = cls.y_test_mnist[0 : cls.n_test]
 
-    def test_keras_mnist(self):
+    def test_6_keras_mnist(self):
         """
         Test with the KerasClassifier. (Untargeted Attack)
         :return:
@@ -66,7 +66,7 @@ class TestThresholdAttack(TestBase):
         classifier = get_image_classifier_kr()
         self._test_attack(classifier, self.x_test_mnist, self.y_test_mnist, False)
 
-    def test_tensorflow_mnist(self):
+    def test_2_tensorflow_mnist(self):
         """
         Test with the TensorFlowClassifier. (Untargeted Attack)
         :return:
@@ -74,7 +74,7 @@ class TestThresholdAttack(TestBase):
         classifier, sess = get_image_classifier_tf()
         self._test_attack(classifier, self.x_test_mnist, self.y_test_mnist, False)
 
-    def test_pytorch_mnist(self):
+    def test_4_pytorch_mnist(self):
         """
         Test with the PyTorchClassifier. (Untargeted Attack)
         :return:
@@ -83,7 +83,7 @@ class TestThresholdAttack(TestBase):
         classifier = get_image_classifier_pt()
         self._test_attack(classifier, x_test, self.y_test_mnist, False)
 
-    def test_keras_mnist_targeted(self):
+    def test_7_keras_mnist_targeted(self):
         """
         Test with the KerasClassifier. (Targeted Attack)
         :return:
@@ -91,7 +91,7 @@ class TestThresholdAttack(TestBase):
         classifier = get_image_classifier_kr()
         self._test_attack(classifier, self.x_test_mnist, self.y_test_mnist, True)
 
-    def test_tensorflow_mnist_targeted(self):
+    def test_3_tensorflow_mnist_targeted(self):
         """
         Test with the TensorFlowClassifier. (Targeted Attack)
         :return:
@@ -99,7 +99,7 @@ class TestThresholdAttack(TestBase):
         classifier, sess = get_image_classifier_tf()
         self._test_attack(classifier, self.x_test_mnist, self.y_test_mnist, True)
 
-    def test_pytorch_mnist_targeted(self):
+    def test_5_pytorch_mnist_targeted(self):
         """
         Test with the PyTorchClassifier. (Targeted Attack)
         :return:
@@ -142,7 +142,7 @@ class TestThresholdAttack(TestBase):
         # Check that x_test has not been modified by attack and classifier
         self.assertAlmostEqual(float(np.max(np.abs(x_test_original - x_test))), 0.0, delta=0.00001)
 
-    def test_classifier_type_check_fail(self):
+    def test_1_classifier_type_check_fail(self):
         backend_test_classifier_type_check_fail(ThresholdAttack, [BaseEstimator, NeuralNetworkMixin, ClassifierMixin])
 
 
