@@ -20,9 +20,6 @@ from __future__ import absolute_import, division, print_function, unicode_litera
 import logging
 import unittest
 
-import keras
-from keras.models import Sequential
-from keras.layers import Dense, Flatten, Conv2D, MaxPooling2D
 import numpy as np
 import tensorflow as tf
 import torch.nn as nn
@@ -61,10 +58,6 @@ class TestMetrics(unittest.TestCase):
         classifier.fit(x_train, y_train, batch_size=BATCH_SIZE, nb_epochs=2, verbose=0)
 
         # Compute minimal perturbations
-        params = {"eps_step": 1.1}
-        emp_robust = empirical_robustness(classifier, x_train, str("fgsm"), params)
-        self.assertEqual(emp_robust, 0.0)
-
         params = {"eps_step": 1.0, "eps": 1.0}
         emp_robust = empirical_robustness(classifier, x_train, str("fgsm"), params)
         self.assertAlmostEqual(emp_robust, 1.000369094488189, 4)
@@ -98,6 +91,21 @@ class TestMetrics(unittest.TestCase):
 
     @staticmethod
     def _cnn_mnist_k(input_shape):
+        import tensorflow as tf
+
+        tf_version = [int(v) for v in tf.__version__.split(".")]
+        if tf_version[0] == 2 and tf_version[1] >= 3:
+            is_tf23_keras24 = True
+            tf.compat.v1.disable_eager_execution()
+            from tensorflow import keras
+            from tensorflow.keras.models import Sequential
+            from tensorflow.keras.layers import Dense, Flatten, Conv2D, MaxPooling2D
+        else:
+            is_tf23_keras24 = False
+            import keras
+            from keras.models import Sequential
+            from keras.layers import Dense, Flatten, Conv2D, MaxPooling2D
+
         # Create simple CNN
         model = Sequential()
         model.add(Conv2D(4, kernel_size=(5, 5), activation="relu", input_shape=input_shape))
@@ -112,41 +120,14 @@ class TestMetrics(unittest.TestCase):
         classifier = KerasClassifier(model=model, clip_values=(0, 1), use_logits=False)
         return classifier
 
-
-#########################################
-# This part is the unit test for Clever.#
-#########################################
-
-
-class Model(nn.Module):
-    def __init__(self):
-        super(Model, self).__init__()
-        self.conv = nn.Conv2d(1, 16, 5)
-        self.pool = nn.MaxPool2d(2, 2)
-        self.fc = nn.Linear(2304, 10)
-
-    def forward(self, x):
-        x = self.pool(f.relu(self.conv(x)))
-        x = x.view(-1, 2304)
-        logit_output = self.fc(x)
-
-        return logit_output
-
-
-class TestClever(unittest.TestCase):
-    """
-    Unittest for Clever metrics.
-    """
-
-    def setUp(self):
-        master_seed(seed=42, set_tensorflow=True)
-
     @staticmethod
     def _create_tfclassifier():
         """
         To create a simple TensorFlowClassifier for testing.
         :return:
         """
+        import tensorflow as tf
+
         # Define input and output placeholders
         input_ph = tf.placeholder(tf.float32, shape=[None, 28, 28, 1])
         labels_ph = tf.placeholder(tf.int32, shape=[None, 10])
@@ -188,6 +169,20 @@ class TestClever(unittest.TestCase):
         To create a simple KerasClassifier for testing.
         :return:
         """
+        import tensorflow as tf
+
+        tf_version = [int(v) for v in tf.__version__.split(".")]
+        if tf_version[0] == 2 and tf_version[1] >= 3:
+            # is_tf23_keras24 = True
+            tf.compat.v1.disable_eager_execution()
+            from tensorflow import keras
+            from tensorflow.keras.models import Sequential
+            from tensorflow.keras.layers import Dense, Flatten, Conv2D, MaxPooling2D
+        else:
+            import keras
+            from keras.models import Sequential
+            from keras.layers import Dense, Flatten, Conv2D, MaxPooling2D
+
         # Create simple CNN
         model = Sequential()
         model.add(Conv2D(4, kernel_size=(5, 5), activation="relu", input_shape=(28, 28, 1)))
@@ -210,6 +205,21 @@ class TestClever(unittest.TestCase):
         To create a simple PyTorchClassifier for testing.
         :return:
         """
+
+        class Model(nn.Module):
+            def __init__(self):
+                super(Model, self).__init__()
+                self.conv = nn.Conv2d(1, 16, 5)
+                self.pool = nn.MaxPool2d(2, 2)
+                self.fc = nn.Linear(2304, 10)
+
+            def forward(self, x):
+                x = self.pool(f.relu(self.conv(x)))
+                x = x.view(-1, 2304)
+                logit_output = self.fc(x)
+
+                return logit_output
+
         # Define the network
         model = Model()
 
@@ -225,7 +235,7 @@ class TestClever(unittest.TestCase):
         return ptc
 
     @unittest.skipIf(tf.__version__[0] == "2", reason="Skip unittests for TensorFlow v2.")
-    def test_clever_tf(self):
+    def test_2_clever_tf(self):
         """
         Test with TensorFlow.
         :return:
@@ -292,7 +302,7 @@ class TestClever(unittest.TestCase):
         self.assertNotEqual(res1, res2)
         self.assertNotEqual(res2, res0)
 
-    def test_clever_pt(self):
+    def test_3_clever_pt(self):
         """
         Test with pytorch.
         :return:
@@ -363,9 +373,7 @@ class TestClever(unittest.TestCase):
         scores = clever(krc, x_test[0], 5, 5, 3, 2, target=np.argmax(krc.predict(x_test[:1])), c_init=1, pool_factor=10)
         self.assertIsNone(scores[0], msg="Clever scores for the predicted class should be `None`.")
 
-
-class TestWassersteinDistance(unittest.TestCase):
-    def test_wasserstein_distance(self):
+    def test_1_wasserstein_distance(self):
         nb_train = 1000
         nb_test = 100
         batch_size = 3
@@ -377,8 +385,9 @@ class TestWassersteinDistance(unittest.TestCase):
 
         wd_0 = wasserstein_distance(x_train[:batch_size], x_train[:batch_size])
         wd_1 = wasserstein_distance(x_train[:batch_size], x_test[:batch_size])
-        wd_2 = wasserstein_distance(x_train[:batch_size], x_train[:batch_size], weights[:batch_size],
-                                    weights[:batch_size])
+        wd_2 = wasserstein_distance(
+            x_train[:batch_size], x_train[:batch_size], weights[:batch_size], weights[:batch_size]
+        )
 
         np.testing.assert_array_equal(wd_0, np.asarray([0.0, 0.0, 0.0]))
         np.testing.assert_array_almost_equal(wd_1, np.asarray([0.04564, 0.01235, 0.04787]), decimal=4)
@@ -388,5 +397,6 @@ class TestWassersteinDistance(unittest.TestCase):
 
         np.testing.assert_array_equal(wd_2, np.asarray([0.0, 0.0, 0.0]))
 
+
 if __name__ == "__main__":
     unittest.main()
