@@ -26,12 +26,13 @@ import os
 import pickle
 import time
 import unittest
+import warnings
 
 import numpy as np
 
-from art.utils import load_dataset
 from art.estimators.encoding.tensorflow import TensorFlowEncoder
 from art.estimators.generation.tensorflow import TensorFlowGenerator
+from art.utils import load_dataset
 
 logger = logging.getLogger(__name__)
 
@@ -275,7 +276,9 @@ def get_image_classifier_tf_v1(from_logits=False, load_init=True, sess=None):
     probabilities = tf.keras.activations.softmax(x=logits)
 
     # Train operator
-    loss = tf.reduce_mean(tf.losses.softmax_cross_entropy(logits=logits, onehot_labels=output_ph))
+    loss = tf.reduce_mean(
+        tf.losses.softmax_cross_entropy(logits=logits, onehot_labels=output_ph, reduction=tf.losses.Reduction.SUM)
+    )
     optimizer = tf.train.AdamOptimizer(learning_rate=0.01)
     train = optimizer.minimize(loss)
 
@@ -377,7 +380,9 @@ def get_image_classifier_tf_v2(from_logits=False):
             )
         )
 
-    loss_object = tf.keras.losses.SparseCategoricalCrossentropy(from_logits=from_logits)
+    loss_object = tf.keras.losses.SparseCategoricalCrossentropy(
+        from_logits=from_logits, reduction=tf.keras.losses.Reduction.SUM
+    )
 
     model.compile(optimizer=optimizer, loss=loss_object)
 
@@ -414,9 +419,20 @@ def get_image_classifier_kr(
     :type load_init: `bool`
     :return: KerasClassifier, tf.Session()
     """
-    import keras
-    from keras.models import Sequential
-    from keras.layers import Dense, Flatten, Conv2D, MaxPooling2D
+    import tensorflow as tf
+
+    tf_version = [int(v) for v in tf.__version__.split(".")]
+    if tf_version[0] == 2 and tf_version[1] >= 3:
+        is_tf23_keras24 = True
+        tf.compat.v1.disable_eager_execution()
+        from tensorflow import keras
+        from tensorflow.keras.models import Sequential
+        from tensorflow.keras.layers import Dense, Flatten, Conv2D, MaxPooling2D
+    else:
+        is_tf23_keras24 = False
+        import keras
+        from keras.models import Sequential
+        from keras.layers import Dense, Flatten, Conv2D, MaxPooling2D
 
     from art.estimators.classification.keras import KerasClassifier
 
@@ -424,16 +440,28 @@ def get_image_classifier_kr(
     model = Sequential()
 
     if load_init:
-        model.add(
-            Conv2D(
-                1,
-                kernel_size=(7, 7),
-                activation="relu",
-                input_shape=(28, 28, 1),
-                kernel_initializer=_kr_weights_loader("MNIST", "W", "CONV2D"),
-                bias_initializer=_kr_weights_loader("MNIST", "B", "CONV2D"),
+        if is_tf23_keras24:
+            model.add(
+                Conv2D(
+                    1,
+                    kernel_size=(7, 7),
+                    activation="relu",
+                    input_shape=(28, 28, 1),
+                    kernel_initializer=_tf_weights_loader("MNIST", "W", "CONV2D", 2),
+                    bias_initializer=_tf_weights_loader("MNIST", "B", "CONV2D", 2),
+                )
+            )
+        else:
+            model.add(
+                Conv2D(
+                    1,
+                    kernel_size=(7, 7),
+                    activation="relu",
+                    input_shape=(28, 28, 1),
+                    kernel_initializer=_kr_weights_loader("MNIST", "W", "CONV2D"),
+                    bias_initializer=_kr_weights_loader("MNIST", "B", "CONV2D"),
+                )
             )
-        )
     else:
         model.add(Conv2D(1, kernel_size=(7, 7), activation="relu", input_shape=(28, 28, 1)))
 
@@ -442,26 +470,46 @@ def get_image_classifier_kr(
 
     if from_logits:
         if load_init:
-            model.add(
-                Dense(
-                    10,
-                    activation="linear",
-                    kernel_initializer=_kr_weights_loader("MNIST", "W", "DENSE"),
-                    bias_initializer=_kr_weights_loader("MNIST", "B", "DENSE"),
+            if is_tf23_keras24:
+                model.add(
+                    Dense(
+                        10,
+                        activation="linear",
+                        kernel_initializer=_tf_weights_loader("MNIST", "W", "DENSE", 2),
+                        bias_initializer=_tf_weights_loader("MNIST", "B", "DENSE", 2),
+                    )
+                )
+            else:
+                model.add(
+                    Dense(
+                        10,
+                        activation="linear",
+                        kernel_initializer=_kr_weights_loader("MNIST", "W", "DENSE"),
+                        bias_initializer=_kr_weights_loader("MNIST", "B", "DENSE"),
+                    )
                 )
-            )
         else:
             model.add(Dense(10, activation="linear"))
     else:
         if load_init:
-            model.add(
-                Dense(
-                    10,
-                    activation="softmax",
-                    kernel_initializer=_kr_weights_loader("MNIST", "W", "DENSE"),
-                    bias_initializer=_kr_weights_loader("MNIST", "B", "DENSE"),
+            if is_tf23_keras24:
+                model.add(
+                    Dense(
+                        10,
+                        activation="softmax",
+                        kernel_initializer=_tf_weights_loader("MNIST", "W", "DENSE", 2),
+                        bias_initializer=_tf_weights_loader("MNIST", "B", "DENSE", 2),
+                    )
+                )
+            else:
+                model.add(
+                    Dense(
+                        10,
+                        activation="softmax",
+                        kernel_initializer=_kr_weights_loader("MNIST", "W", "DENSE"),
+                        bias_initializer=_kr_weights_loader("MNIST", "B", "DENSE"),
+                    )
                 )
-            )
         else:
             model.add(Dense(10, activation="softmax"))
 
@@ -485,7 +533,7 @@ def get_image_classifier_kr(
 
                     loss = categorical_crossentropy
                 else:
-                    raise AttributeError("This combination of loss function options is not supported.")
+                    raise NotImplementedError("This combination of loss function options is not supported.")
             else:
                 loss = keras.losses.categorical_crossentropy
         elif loss_type == "function_backend":
@@ -550,6 +598,98 @@ def get_image_classifier_kr(
     return krc
 
 
+def get_image_classifier_kr_functional(input_layer=1, output_layer=1):
+    from art.estimators.classification.keras import KerasClassifier
+    import keras
+    from keras.layers import Conv2D, Dense, Dropout, Flatten, Input, MaxPooling2D
+    from keras.models import Model
+
+    def _functional_model():
+        in_layer = Input(shape=(28, 28, 1), name="input0")
+        layer = Conv2D(32, kernel_size=(3, 3), activation="relu")(in_layer)
+        layer = Conv2D(64, (3, 3), activation="relu")(layer)
+        layer = MaxPooling2D(pool_size=(2, 2))(layer)
+        layer = Dropout(0.25)(layer)
+        layer = Flatten()(layer)
+        layer = Dense(128, activation="relu")(layer)
+        layer = Dropout(0.5)(layer)
+        out_layer = Dense(10, activation="softmax", name="output0")(layer)
+
+        in_layer_2 = Input(shape=(28, 28, 1), name="input1")
+        layer = Conv2D(32, kernel_size=(3, 3), activation="relu")(in_layer_2)
+        layer = Conv2D(64, (3, 3), activation="relu")(layer)
+        layer = MaxPooling2D(pool_size=(2, 2))(layer)
+        layer = Dropout(0.25)(layer)
+        layer = Flatten()(layer)
+        layer = Dense(128, activation="relu")(layer)
+        layer = Dropout(0.5)(layer)
+        out_layer_2 = Dense(10, activation="softmax", name="output1")(layer)
+
+        model = Model(inputs=[in_layer, in_layer_2], outputs=[out_layer, out_layer_2])
+
+        model.compile(
+            loss=keras.losses.categorical_crossentropy,
+            optimizer=keras.optimizers.Adadelta(),
+            metrics=["accuracy"],
+            loss_weights=[1.0, 1.0],
+        )
+
+        return model
+
+    functional_model = _functional_model()
+
+    return KerasClassifier(functional_model, clip_values=(0, 1), input_layer=input_layer, output_layer=output_layer)
+
+
+def get_image_classifier_kr_tf_functional(input_layer=1, output_layer=1):
+    """
+       Standard Keras_tf classifier for unit testing built with a functional model
+
+       :return: KerasClassifier
+       """
+    import tensorflow as tf
+
+    if tf.__version__[0] == "2":
+        tf.compat.v1.disable_eager_execution()
+    from tensorflow.keras.layers import Conv2D, Dense, Dropout, Flatten, Input, MaxPooling2D
+    from tensorflow.keras.models import Model
+    from art.estimators.classification.keras import KerasClassifier
+
+    def functional_model():
+        in_layer = Input(shape=(28, 28, 1), name="input0")
+        layer = Conv2D(32, kernel_size=(3, 3), activation="relu")(in_layer)
+        layer = Conv2D(64, (3, 3), activation="relu")(layer)
+        layer = MaxPooling2D(pool_size=(2, 2))(layer)
+        layer = Dropout(0.25)(layer)
+        layer = Flatten()(layer)
+        layer = Dense(128, activation="relu")(layer)
+        layer = Dropout(0.5)(layer)
+        out_layer = Dense(10, activation="softmax", name="output0")(layer)
+
+        in_layer_2 = Input(shape=(28, 28, 1), name="input1")
+        layer = Conv2D(32, kernel_size=(3, 3), activation="relu")(in_layer_2)
+        layer = Conv2D(64, (3, 3), activation="relu")(layer)
+        layer = MaxPooling2D(pool_size=(2, 2))(layer)
+        layer = Dropout(0.25)(layer)
+        layer = Flatten()(layer)
+        layer = Dense(128, activation="relu")(layer)
+        layer = Dropout(0.5)(layer)
+        out_layer_2 = Dense(10, activation="softmax", name="output1")(layer)
+
+        model = Model(inputs=[in_layer, in_layer_2], outputs=[out_layer, out_layer_2])
+
+        model.compile(
+            loss=tf.keras.losses.categorical_crossentropy,
+            optimizer=tf.keras.optimizers.Adadelta(),
+            metrics=["accuracy"],
+            loss_weights=[1.0, 1.0],
+        )
+
+        return model
+
+    return KerasClassifier(functional_model(), clip_values=(0, 1), input_layer=input_layer, output_layer=output_layer)
+
+
 def get_image_classifier_kr_tf(loss_name="categorical_crossentropy", loss_type="function", from_logits=False):
     """
     Standard Keras classifier for unit testing
@@ -714,7 +854,7 @@ def get_image_classifier_kr_tf(loss_name="categorical_crossentropy", loss_type="
 
 def get_image_classifier_kr_tf_binary():
     """
-    Standard Tensorflow-Keras binary classifier for unit testing
+    Standard TensorFlow-Keras binary classifier for unit testing
 
     :return: KerasClassifier
     """
@@ -752,7 +892,7 @@ def get_image_classifier_kr_tf_binary():
 
 def get_image_classifier_kr_tf_with_wildcard():
     """
-    Standard Tensorflow-Keras binary classifier for unit testing
+    Standard TensorFlow-Keras binary classifier for unit testing
 
     :return: KerasClassifier
     """
@@ -790,7 +930,8 @@ def get_image_classifier_pt(from_logits=False, load_init=True):
     :return: PyTorchClassifier
     """
     import torch
-
+    import torch.nn as nn
+    import torch.optim as optim
     from art.estimators.classification.pytorch import PyTorchClassifier
 
     class Model(torch.nn.Module):
@@ -804,6 +945,7 @@ def get_image_classifier_pt(from_logits=False, load_init=True):
             super(Model, self).__init__()
 
             self.conv = torch.nn.Conv2d(in_channels=1, out_channels=1, kernel_size=7)
+            self.relu = torch.nn.ReLU()
             self.pool = torch.nn.MaxPool2d(4, 4)
             self.fullyconnected = torch.nn.Linear(25, 10)
 
@@ -845,7 +987,7 @@ def get_image_classifier_pt(from_logits=False, load_init=True):
             :return: Prediction of the model
             """
             x = self.conv(x)
-            x = torch.nn.functional.relu(x)
+            x = self.relu(x)
             x = self.pool(x)
             x = x.reshape(-1, 25)
             x = self.fullyconnected(x)
@@ -857,7 +999,7 @@ def get_image_classifier_pt(from_logits=False, load_init=True):
     model = Model()
 
     # Define a loss function and optimizer
-    loss_fn = torch.nn.CrossEntropyLoss()
+    loss_fn = torch.nn.CrossEntropyLoss(reduction="sum")
     optimizer = torch.optim.Adam(model.parameters(), lr=0.01)
 
     # Get classifier
@@ -877,7 +1019,7 @@ def get_classifier_bb(defences=None):
     from art.estimators.classification.blackbox import BlackBoxClassifier
     from art.utils import to_categorical
 
-    # define blackbox classifier
+    # define black-box classifier
     def predict(x):
         with open(
             os.path.join(os.path.dirname(os.path.dirname(__file__)), "utils/data/mnist", "api_output.txt")
@@ -889,51 +1031,44 @@ def get_classifier_bb(defences=None):
     return bbc
 
 
-def get_classifier_mx():
-    """
-    Standard MXNet classifier for unit testing
-
-    :return: MXNetClassifier
-    """
+def get_image_classifier_mxnet_custom_ini():
     import mxnet
-    from mxnet.gluon.nn import Conv2D, MaxPool2D, Flatten, Dense
-    from art.estimators.classification import MXClassifier
 
-    model = mxnet.gluon.nn.Sequential()
-    with model.name_scope():
-        model.add(
-            Conv2D(channels=1, kernel_size=7, activation="relu"),
-            MaxPool2D(pool_size=4, strides=4),
-            Flatten(),
-            Dense(10),
-        )
-    model.initialize(init=mxnet.init.Xavier())
-
-    # Create optimizer
-    loss = mxnet.gluon.loss.SoftmaxCrossEntropyLoss()
-    trainer = mxnet.gluon.Trainer(model.collect_params(), "sgd", {"learning_rate": 0.1})
-
-    # # Fit classifier
-    # classifier = MXClassifier(model=net, loss=loss, clip_values=(0, 1), input_shape=(1, 28, 28), nb_classes=10,
-    #                           optimizer=trainer)
-    # classifier.fit(x_train, y_train, batch_size=128, nb_epochs=2)
-    # cls.classifier = classifier
-
-    # Get classifier
-    mxc = MXClassifier(
-        model=model,
-        loss=loss,
-        input_shape=(28, 28, 1),
-        nb_classes=10,
-        optimizer=trainer,
-        ctx=None,
-        channels_first=True,
-        clip_values=(0, 1),
-        defences=None,
-        preprocessing=(0, 1),
+    w_conv2d = np.load(
+        os.path.join(os.path.dirname(os.path.dirname(__file__)), "utils/resources/models", "W_CONV2D_MNIST.npy")
+    )
+    b_conv2d = np.load(
+        os.path.join(os.path.dirname(os.path.dirname(__file__)), "utils/resources/models", "B_CONV2D_MNIST.npy")
+    )
+    w_dense = np.load(
+        os.path.join(os.path.dirname(os.path.dirname(__file__)), "utils/resources/models", "W_DENSE_MNIST.npy")
+    )
+    b_dense = np.load(
+        os.path.join(os.path.dirname(os.path.dirname(__file__)), "utils/resources/models", "B_DENSE_MNIST.npy")
     )
 
-    return mxc
+    w_conv2d_mx = w_conv2d.reshape((1, 1, 7, 7))
+
+    alias = mxnet.registry.get_alias_func(mxnet.initializer.Initializer, "initializer")
+
+    @mxnet.init.register
+    @alias("mm_init")
+    class CustomInit(mxnet.init.Initializer):
+        def __init__(self):
+            super(CustomInit, self).__init__()
+            self.params = dict()
+            self.params["conv0_weight"] = w_conv2d_mx
+            self.params["conv0_bias"] = b_conv2d
+            self.params["dense0_weight"] = np.transpose(w_dense)
+            self.params["dense0_bias"] = b_dense
+
+        def _init_weight(self, name, arr):
+            arr[:] = self.params[name]
+
+        def _init_bias(self, name, arr):
+            arr[:] = self.params[name]
+
+    return CustomInit()
 
 
 def get_gan_inverse_gan_ft():
@@ -1144,48 +1279,67 @@ def get_tabular_classifier_tf_v2():
     return tfc
 
 
-def get_tabular_classifier_scikit_list(clipped=False):
-    model_list_names = [
-        "decisionTreeClassifier",
-        "extraTreeClassifier",
-        "adaBoostClassifier",
-        "baggingClassifier",
-        "extraTreesClassifier",
-        "gradientBoostingClassifier",
-        "randomForestClassifier",
-        "logisticRegression",
-        "svc",
-        "linearSVC",
-    ]
-    if clipped:
-        classifier_list = [
-            # os.path.join(os.path.dirname(os.path.dirname(__file__)),'utils/resources/models', 'W_DENSE3_IRIS.npy')
-            pickle.load(
-                open(
-                    os.path.join(
-                        os.path.dirname(os.path.dirname(__file__)),
-                        "utils/resources/models/scikit/",
-                        model_name + "iris_clipped.sav",
-                    ),
-                    "rb",
-                )
-            )
-            for model_name in model_list_names
-        ]
+def get_tabular_classifier_scikit_list(clipped=False, model_list_names=None):
+    from art.estimators.classification.scikitlearn import (
+        ScikitlearnDecisionTreeClassifier,
+        # ScikitlearnExtraTreeClassifier,
+        ScikitlearnAdaBoostClassifier,
+        ScikitlearnBaggingClassifier,
+        ScikitlearnExtraTreesClassifier,
+        ScikitlearnGradientBoostingClassifier,
+        ScikitlearnRandomForestClassifier,
+        ScikitlearnLogisticRegression,
+        ScikitlearnSVC,
+    )
+
+    available_models = {
+        "decisionTreeClassifier": ScikitlearnDecisionTreeClassifier,
+        # "extraTreeClassifier": ScikitlearnExtraTreeClassifier,
+        "adaBoostClassifier": ScikitlearnAdaBoostClassifier,
+        "baggingClassifier": ScikitlearnBaggingClassifier,
+        "extraTreesClassifier": ScikitlearnExtraTreesClassifier,
+        "gradientBoostingClassifier": ScikitlearnGradientBoostingClassifier,
+        "randomForestClassifier": ScikitlearnRandomForestClassifier,
+        "logisticRegression": ScikitlearnLogisticRegression,
+        "svc": ScikitlearnSVC,
+        "linearSVC": ScikitlearnSVC,
+    }
+
+    if model_list_names is None:
+        model_dict_names = available_models
     else:
-        classifier_list = [
-            pickle.load(
+        model_dict_names = dict()
+        for name in model_list_names:
+            model_dict_names[name] = available_models[name]
+
+    classifier_list = list()
+
+    if clipped:
+        for model_name, model_class in model_dict_names.items():
+            model = pickle.load(
                 open(
                     os.path.join(
                         os.path.dirname(os.path.dirname(__file__)),
                         "utils/resources/models/scikit/",
-                        model_name + "iris_unclipped.sav",
+                        "scikit-" + model_name + "-iris-clipped.pickle",
                     ),
                     "rb",
                 )
             )
-            for model_name in model_list_names
-        ]
+            classifier_list.append(model_class(model=model, clip_values=(0, 1)))
+    else:
+        for model_name, model_class in model_dict_names.items():
+            model = pickle.load(
+                open(
+                    os.path.join(
+                        os.path.dirname(os.path.dirname(__file__)),
+                        "utils/resources/models/scikit/",
+                        "scikit-" + model_name + "-iris-unclipped.pickle",
+                    ),
+                    "rb",
+                )
+            )
+            classifier_list.append(model_class(model=model, clip_values=None))
 
     return classifier_list
 
@@ -1200,9 +1354,20 @@ def get_tabular_classifier_kr(load_init=True):
     :return: The trained model for Iris dataset and the session.
     :rtype: `tuple(KerasClassifier, tf.Session)`
     """
-    import keras
-    from keras.models import Sequential
-    from keras.layers import Dense
+    import tensorflow as tf
+
+    tf_version = [int(v) for v in tf.__version__.split(".")]
+    if tf_version[0] == 2 and tf_version[1] >= 3:
+        is_tf23_keras24 = True
+        tf.compat.v1.disable_eager_execution()
+        from tensorflow import keras
+        from tensorflow.keras.models import Sequential
+        from tensorflow.keras.layers import Dense
+    else:
+        is_tf23_keras24 = False
+        import keras
+        from keras.models import Sequential
+        from keras.layers import Dense
 
     from art.estimators.classification.keras import KerasClassifier
 
@@ -1210,31 +1375,58 @@ def get_tabular_classifier_kr(load_init=True):
     model = Sequential()
 
     if load_init:
-        model.add(
-            Dense(
-                10,
-                input_shape=(4,),
-                activation="relu",
-                kernel_initializer=_kr_weights_loader("IRIS", "W", "DENSE1"),
-                bias_initializer=_kr_weights_loader("IRIS", "B", "DENSE1"),
+        if is_tf23_keras24:
+            model.add(
+                Dense(
+                    10,
+                    input_shape=(4,),
+                    activation="relu",
+                    kernel_initializer=_tf_weights_loader("IRIS", "W", "DENSE1", 2),
+                    bias_initializer=_tf_weights_loader("IRIS", "B", "DENSE1", 2),
+                )
             )
-        )
-        model.add(
-            Dense(
-                10,
-                activation="relu",
-                kernel_initializer=_kr_weights_loader("IRIS", "W", "DENSE2"),
-                bias_initializer=_kr_weights_loader("IRIS", "B", "DENSE2"),
+            model.add(
+                Dense(
+                    10,
+                    activation="relu",
+                    kernel_initializer=_tf_weights_loader("IRIS", "W", "DENSE2", 2),
+                    bias_initializer=_tf_weights_loader("IRIS", "B", "DENSE2", 2),
+                )
             )
-        )
-        model.add(
-            Dense(
-                3,
-                activation="softmax",
-                kernel_initializer=_kr_weights_loader("IRIS", "W", "DENSE3"),
-                bias_initializer=_kr_weights_loader("IRIS", "B", "DENSE3"),
+            model.add(
+                Dense(
+                    3,
+                    activation="softmax",
+                    kernel_initializer=_tf_weights_loader("IRIS", "W", "DENSE3", 2),
+                    bias_initializer=_tf_weights_loader("IRIS", "B", "DENSE3", 2),
+                )
+            )
+        else:
+            model.add(
+                Dense(
+                    10,
+                    input_shape=(4,),
+                    activation="relu",
+                    kernel_initializer=_kr_weights_loader("IRIS", "W", "DENSE1"),
+                    bias_initializer=_kr_weights_loader("IRIS", "B", "DENSE1"),
+                )
+            )
+            model.add(
+                Dense(
+                    10,
+                    activation="relu",
+                    kernel_initializer=_kr_weights_loader("IRIS", "W", "DENSE2"),
+                    bias_initializer=_kr_weights_loader("IRIS", "B", "DENSE2"),
+                )
+            )
+            model.add(
+                Dense(
+                    3,
+                    activation="softmax",
+                    kernel_initializer=_kr_weights_loader("IRIS", "W", "DENSE3"),
+                    bias_initializer=_kr_weights_loader("IRIS", "B", "DENSE3"),
+                )
             )
-        )
     else:
         model.add(Dense(10, input_shape=(4,), activation="relu"))
         model.add(Dense(10, activation="relu"))
@@ -1248,6 +1440,19 @@ def get_tabular_classifier_kr(load_init=True):
     return krc
 
 
+class ARTTestException(Exception):
+    def __init__(self, message):
+        super().__init__(message)
+
+
+class ARTTestFixtureNotImplemented(ARTTestException):
+    def __init__(self, message, fixture_name, framework, parameters_dict=""):
+        super().__init__(
+            "Could NOT run test for framework: {0} due to fixture: {1}. Message was: '"
+            "{2}' for the following parameters: {3}".format(framework, fixture_name, message, parameters_dict)
+        )
+
+
 def get_tabular_classifier_pt(load_init=True):
     """
     Standard PyTorch classifier for unit testing on Iris dataset.
@@ -1344,6 +1549,41 @@ def get_tabular_classifier_pt(load_init=True):
     return ptc
 
 
+def get_attack_classifier_pt(num_features):
+    """
+    PyTorch classifier for testing membership inference attacks.
+
+    :param num_features: The number of features in the attack model.
+    :type num_features: `int`
+    :return: Model for attack.
+    :rtype: :class:`.PyTorchClassifier`
+    """
+    import torch.nn as nn
+    import torch.optim as optim
+    from art.estimators.classification.pytorch import PyTorchClassifier
+
+    class AttackModel(nn.Module):
+        def __init__(self, num_features):
+            super(AttackModel, self).__init__()
+            self.layer = nn.Linear(num_features, 1)
+            self.output = nn.Sigmoid()
+
+        def forward(self, x):
+            return self.output(self.layer(x))
+
+    # Create model
+    model = AttackModel(num_features)
+
+    # Define a loss function and optimizer
+    loss_fn = nn.BCELoss()
+    optimizer = optim.Adam(model.parameters(), lr=0.0001)
+    attack_model = PyTorchClassifier(
+        model=model, loss=loss_fn, optimizer=optimizer, input_shape=(num_features,), nb_classes=1
+    )
+
+    return attack_model
+
+
 # -------------------------------------------------------------------------------------------- RANDOM NUMBER GENERATORS
 
 
